import asyncio
from typing import Union

import aiohttp
from ..poe.config import poe_persistor
from ..chatgpt_web.config import gptweb_persistor
from nonebot_plugin_guild_patch import GuildMessageEvent
from nonebot.adapters.onebot.v11 import Message, MessageEvent, MessageSegment


def is_auto_prompt(prompt_nickname):
    if prompt_nickname in [gptweb_persistor.auto_prompt, poe_persistor.auto_prompt]:
        if prompt_nickname in gptweb_persistor.auto_prompt:
            return "CharGPT_Web"
        elif prompt_nickname in poe_persistor.auto_prompt:
            return "Poe"
    else:
        return False


def reply_out(
    event: MessageEvent, content: Union[MessageSegment, Message, str, bytes]
) -> Message:
    """返回一个回复消息"""
    if isinstance(event, GuildMessageEvent):
        return Message(content)
    if type(content) == bytes:
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    if content[0:9] == "base64://":
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    return MessageSegment.reply(event.message_id) + content


async def get_url(text):
    """将 Markdown 文本保存到 Mozilla Pastebin，并获得 URL"""
    async with aiohttp.ClientSession() as session:
        payload = {
            "expires": "86400",
            "format": "url",
            "lexer": "_markdown",
            "content": text,
        }
        retries = 3
        while retries > 0:
            try:
                async with session.post(
                    "https://pastebin.mozilla.org/api/", data=payload
                ) as resp:
                    resp.raise_for_status()
                    url = await resp.text()
                    url = url[0:-1]
                    return url
            except Exception as e:
                retries -= 1
                if retries == 0:
                    url = f"上传失败：{str(e)}"
                    return url
                await asyncio.sleep(1)



import asyncio
import json
import re
from playwright.async_api import ElementHandle
from playwright.sync_api import Page
from .config import poe_persistor


async def send_message_async(page: Page, botname: str, input_str: str):
    # 定义重试次数和重试间隔时间
    retry_count = 5
    retry_interval = 0.5

    # 定义当前重试次数和报错次数
    current_retry = 0
    error_count = 0

    while current_retry < retry_count:
        try:
            # 打开目标网页
            await page.goto(f"https://poe.com/{botname}")
            text = "This bot has been deleted for violating our"
            content = await page.content()
            if text in content:
                return "banned"
            if "0 free" in content:
                return "limited"
            # 找到输入框元素
            input_box = await page.wait_for_selector(
                ".ChatMessageInputView_textInput__Aervw", timeout=2000
            )

            # 将字符串发送到输入框中
            await input_box.fill(input_str)

            # 找到发送按钮元素并点击,容易点不到，多等一次
            await page.wait_for_selector("button.Button_primary__pIDjn", timeout=1000)
            send_button = await page.wait_for_selector(
                "button.Button_primary__pIDjn", timeout=1000
            )
            # await asyncio.sleep(0.5)
            await send_button.click()
            break
        except:
            # 如果出现超时异常，打印错误信息并稍等一段时间后重试
            error_count += 1
            if error_count >= 1:
                await asyncio.sleep(retry_interval)
            current_retry += 1
            continue
    if current_retry == retry_count:
        return False


async def get_message_async(page, botname, sleep, nosuggest=False):
    consecutive_errors = 0
    answer_lost = 0
    suggest_lost = 0
    while True:
        await asyncio.sleep(sleep)
        try:
            await page.reload()
            response = await page.content()
            text = "This bot has been deleted for violating"
            if text in response:
                return "banned"
            if "Message_errorBubble" in response:
                answer_lost += 1
            if answer_lost > 2:
                return False
            match_text = re.search(
                r'<script id="__NEXT_DATA__" type="application/json">*(.*?)</script>',
                response,
                re.DOTALL,
            )
            json_data_text = match_text.group(1)
            json_obj_text = json.loads(json_data_text)

            chat_list_raw = json_obj_text["props"]["pageProps"]["payload"][
                "chatOfBotDisplayName"
            ]["messagesConnection"]["edges"]

            chat_list_raw = [a["node"] for a in chat_list_raw]
            chat_list_text = [a["text"] for a in chat_list_raw]
            if chat_list_text[-1]:
                if nosuggest:
                    return chat_list_text, []
                else:
                    match_suggest = re.search(
                        r'<section class="ChatMessageSuggestedReplies_suggestedRepliesContainer__JgW12">*(.*?)</section>',
                        response,
                        re.DOTALL,
                    )
                    string_list = re.findall(
                        r">\s*([^<>\n]+)\s*<", match_suggest.group(1)
                    )
                    if len(string_list) >= 4:
                        return chat_list_text, string_list[0:4]
                    if len(string_list) == 1:
                        suggest_lost += 1
                    if suggest_lost > 2:
                        return chat_list_text, string_list
        except:
            consecutive_errors += 1
            print(f"poe-chat：获取返回消息失败{consecutive_errors}次")
            if consecutive_errors >= 5:
                return False
        else:
            consecutive_errors = 0


# 发送并接受
async def poe_chat(botname, question, page, nosuggest=False):
    if botname == "NeevaAI":
        nosuggest = True
    result1 = await send_message_async(page, botname, question)
    if result1 == "banned":
        return "banned"
    elif result1 == "limited":
        return "limited"
    elif result1 == False:
        return False
    if poe_persistor.suggest_able == "False":
        nosuggest = True
    result2 = await get_message_async(page, botname, sleep=2, nosuggest=nosuggest)
    if isinstance(result2, tuple):
        answers, suggests = result2
        return answers[-1], suggests
    elif isinstance(result2, str):
        is_banned = result2
        return "banned"
    elif isinstance(result2, bool):
        is_got = result2
        return is_got


# 清空聊天记录
async def poe_clear(page, truename):
    try:
        await page.goto(f"https://poe.com/{truename}")

        # 等待元素出现
        element = await page.wait_for_selector(
            "div.ChatMessageInputView_paintbrushWraper__DHMNW"
        )
        await asyncio.sleep(1)
        await element.wait_for_selector(
            "svg.ChatMessageInputView_paintbrushIcon__Turkx"
        )
        svg_element = await page.query_selector(
            ".ChatMessageInputView_paintbrushIcon__Turkx"
        )

        # Get the bounding box of the SVG element
        bbox = await svg_element.bounding_box()
        x = bbox["x"] + 10  # Replace with your desired x coordinate
        y = bbox["y"] - 10  # Replace with your desired y coordinate
        await page.mouse.click(x=x, y=y)

        return True
    except:
        return False


# 创建机器人
async def poe_create(page, botname, base_bot_index, prompt, retries=2):
    try:
        for i in range(retries):
            await page.goto("https://poe.com/create_bot")

            # 定位输入框并清空原有的默认值
            name_input: ElementHandle = await page.wait_for_selector(
                'input[name="name"]'
            )
            await name_input.fill("")

            # 输入新的值
            await name_input.fill(botname)

            await page.wait_for_selector('select[name="baseBot"]')

            if base_bot_index == 1:
                value = "chinchilla"
            elif base_bot_index == 2:
                value = "a2"
            # 根据索引选择选项
            await page.select_option(".Select_select__I0JvU", value=value)

            # 添加预设
            prompt_textarea = await page.wait_for_selector('textarea[name="prompt"]')
            await prompt_textarea.fill(prompt)

            chevron_button = await page.wait_for_selector(
                ".BotInfoForm_chevronDown__LFWWC"
            )
            await chevron_button.click()

            # 点击"Suggest replies"复选框的<label>元素
            checkbox_label: ElementHandle = await page.wait_for_selector(
                '//div[contains(text(), "Suggest replies")]/following-sibling::label'
            )
            await checkbox_label.click()

            # 点击"Linkify bot responses"复选框的<label>元素
            checkbox_label = await page.wait_for_selector(
                '//div[contains(text(), "Linkify bot responses")]/following-sibling::label'
            )
            await checkbox_label.click()

            # 点击"Create bot"按钮
            create_bot_button = await page.wait_for_selector(
                "button.Button_primary__pIDjn"
            )
            await create_bot_button.click()

            # 等待新页面加载完成
            try:
                await page.wait_for_selector(
                    "textarea.ChatMessageInputView_textInput__Aervw", timeout=5000
                )
                return True
            except:
                pass

        return False
    except:
        return False


async def submit_email(page, email):
    for i in range(5):
        try:
            await page.goto("https://poe.com")
            # 点击 "Use email" 按钮
            use_email_button = await page.query_selector('button:has-text("Use email")')
            await use_email_button.click()

            # 填写 email 地址
            email_input = await page.wait_for_selector(
                "input.EmailInput_emailInput__4v_bn"
            )
            await email_input.fill(email)

            # 点击 "Go" 按钮
            go_button = await page.query_selector('button:has-text("Go")')
            await go_button.click()

            # 等待跳转并检查输入框是否存在
            await page.wait_for_selector(
                "input.VerificationCodeInput_verificationCodeInput__YD3KV"
            )
            return True
        except:
            await page.reload()
    return False


async def submit_code(page, code, path):
    retry_count = 0
    while retry_count < 3:
        try:
            # 填写验证码
            code_input = await page.wait_for_selector(
                "input.VerificationCodeInput_verificationCodeInput__YD3KV"
            )
            await code_input.fill("")  # 清空输入框
            await code_input.fill(code)

            # 点击 "Log In" 按钮
            login_button = await page.query_selector('button:has-text("Log In")')
            await login_button.click()

            # 等待页面跳转，并检查页面是否有指定的输入框元素
            await page.wait_for_selector(
                "textarea.ChatMessageInputView_textInput__Aervw"
            )

            # 获取当前页面的 cookie，并保存 poe.com 域名下的 cookie 到本地文件中
            cookies = await page.context.cookies()
            poe_cookies = [
                cookie for cookie in cookies if cookie["domain"] == "poe.com"
            ]
            if len(poe_cookies) > 0:
                with open(path, "w") as f:
                    json.dump(poe_cookies[0], f)

            return True
        except:
            # 如果页面上没有指定的输入框元素，就认为是失败了
            if not await page.query_selector(
                "textarea.ChatMessageInputView_textInput__Aervw"
            ):
                retry_count += 1
                # 等待一段时间后再次尝试输入验证码并点击 "Log In" 按钮
                await asyncio.sleep(1)
            else:
                raise
    return False



import nonebot

from .poe import main
from .newbing import main
from .chatgpt_web import main
from .common import main
from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import (
    Event,
    Bot,
    MessageSegment,
)

from .common.render.render import md_to_pic

poe_help = nonebot.on_command(
    "help", aliases={"帮助", "使用说明", "说明"}, priority=4, block=False
)


@poe_help.handle()
async def __poe_help__(bot: Bot, matcher: Matcher, event: Event):
    user_id = str(event.user_id)
    msg = """
# Spark-GPT 帮助说明

- 以下命令前面全部要加 '/'。

## Poe帮助命令

| 命令 | 描述 |
| --- | --- |
| `/poehelp / ph` | 获取Poe帮助说明。 |

## Newbing帮助命令

| 命令 | 描述 |
| --- | --- |
| `/binghelp / bh` | 获取Newbing帮助说明。 |

## GPT_Web帮助命令

| 命令 | 描述 |
| --- | --- |
| `/gwhelp / gwh` | 获取GPT_Web帮助说明。 |

# 通用命令

- 所有用户均可使用

| 命令 | 描述 |
| --- | --- |
| `/botlist / bl` | 获取你的所有机器人的列表。 |

- 只有spark管理员可以使用

| 命令 | 描述 |
| --- | --- |
| `/添加预设 / ap` | 添加通用预设 |
| `/删除预设 / rp` | 删除通用预设 |

"""
    pic = await md_to_pic(msg)
    await poe_help.send(MessageSegment.image(pic))
    await poe_help.finish()



import json
from pathlib import Path
import re
from nonebot import logger
from nonebot.plugin import on_command, on_message
from nonebot.params import ArgStr, CommandArg
from nonebot.typing import T_State
from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import (
    Message,
    Event,
    Bot,
    MessageEvent,
    MessageSegment,
)
from ImageGen import ImageGenAsync
from .newbing_func import is_useable, reply_out, sendmsg
from .config import newbing_persistor, NewBingTemper, set_userdata
from EdgeGPT import Chatbot, ConversationStyle
from ..common.render.render import md_to_pic

logger.info("开始加载Newbing")
newbingtemper = NewBingTemper()
user_data_dict = newbingtemper.user_data_dict


#############################################################
async def _is_chat_(event: MessageEvent, bot: Bot):
    if is_useable(event):
        try:
            current_userinfo, current_userdata = set_userdata(
                event=event, user_data_dict=user_data_dict
            )
        except:
            return False
        if bool(event.reply):
            if (
                str(event.reply.sender.user_id) == bot.self_id
                and event.reply.message_id == current_userdata.last_reply_message_id
            ):
                chatbot = current_userdata.chatbot
                raw_message = str(event.message)
                if not raw_message:
                    return False
                else:
                    return raw_message, chatbot, current_userinfo, current_userdata
            else:
                return False
        elif str(event.message).startswith(("/bing", "/b")) and not str(
            event.message
        ).startswith(
            (
                "/bc",
                "/bingchange",
                "/bing切换",
                "/bingdraw",
                "/bd",
                "/bing绘图",
                "/bing画图",
                "/binghelp",
                "/bing帮助",
                "/bh",
                "/bl",
                "/bot",
            )
        ):
            raw_message = (
                str(event.message)
                .replace("/bing ", "")
                .replace("/bing", "")
                .replace("/b ", "")
                .replace("/b", "")
            )
            if current_userdata.chatbot:
                chatbot = current_userdata.chatbot
            else:
                try:
                    if len(newbing_persistor.proxy) > 0:
                        chatbot = await Chatbot.create(
                            cookie_path=newbing_persistor.cookie_path_,
                            proxy=newbing_persistor.proxy,
                        )
                    else:
                        chatbot = await Chatbot.create(
                            cookie_path=newbing_persistor.cookie_path_
                        )
                    current_userdata.chatbot = chatbot
                except FileNotFoundError:
                    logger.warning("newbing cookie未配置,无法使用，跳过")
                    return False
            if not raw_message:
                return False
            else:
                return raw_message, chatbot, current_userinfo, current_userdata
    else:
        return False


newbing_chat = on_message(priority=1, block=False)


@newbing_chat.handle()
async def __newbing_chat__(matcher: Matcher, event: Event, bot: Bot):
    temp_tuple = await _is_chat_(event, bot)
    if temp_tuple == False or not temp_tuple:
        await matcher.finish()
    else:
        if isinstance(temp_tuple, tuple):
            raw_message, chatbot, current_userinfo, current_userdata = temp_tuple
        elif isinstance(temp_tuple, str):
            await matcher.finish(reply_out(event, f"出错了:{temp_tuple}"))

    if raw_message in [
        "清除对话",
        "清空对话",
        "清除历史",
        "清空历史",
        "清除记录",
        "清空记录",
        "清空历史对话",
        "刷新对话",
    ]:
        if len(newbing_persistor.proxy) > 0:
            chatbot = await Chatbot.create(
                cookie_path=newbing_persistor.cookie_path_,
                proxy=newbing_persistor.proxy,
            )
        else:
            chatbot = await Chatbot.create(cookie_path=newbing_persistor.cookie_path_)
        current_userdata.chatbot = chatbot
        await matcher.finish(reply_out(event, "成功刷新对话"))
    if (
        len(raw_message) == 1
        and raw_message in ["1", "2", "3"]
        and current_userdata.last_suggests
    ):
        raw_message = current_userdata.last_suggests[int(raw_message) - 1]
    if current_userdata.is_waiting:
        await matcher.finish(reply_out(event, "你有一个对话进行中，请等结束后再继续询问"))
    chatmode = current_userdata.chatmode
    if chatmode == "1":
        style = ConversationStyle.creative
    elif chatmode == "2":
        style = ConversationStyle.balanced
    else:
        style = ConversationStyle.precise
    current_userdata.is_waiting = True
    try:
        raw_json = await chatbot.ask(
            prompt=raw_message,
            conversation_style=style,
            wss_link="wss://sydney.bing.com/sydney/ChatHub",
        )
    except:
        current_userdata.is_waiting = False
        await matcher.send(reply_out(event, "出错喽，多次重试都出错的话，联系机器人主人"))
        await matcher.finish()
    value = raw_json["item"]["result"]["value"]
    if value != "Success":
        if value == "Throttled":
            current_userdata.is_waiting = False
            await matcher.finish(reply_out(event, "该账号cookie问答次数已达到今日上限捏"))
        elif value == "InvalidSession":
            current_userdata.is_waiting = False
            await matcher.finish(reply_out(event, "无效会话捏"))
        else:
            current_userdata.is_waiting = False
            await matcher.finish(reply_out(event, "出错喽，多次重试都出错的话，联系机器人主人"))
    try:
        is_offense = raw_json["item"]["messages"][0]["offense"]

        if is_offense == "Offensive":
            current_userdata.is_waiting = False
            await matcher.finish(reply_out(event, "你的询问太冒犯了,newbing拒绝回答"))
        if "hiddenText" in raw_json["item"]["messages"][1]:
            current_userdata.is_waiting = False
            await matcher.finish(reply_out(event, "你的询问太敏感了,newbing拒绝回答"))
    except:
        pass

    try:
        max_num = raw_json["item"]["throttling"]["maxNumUserMessagesInConversation"]
        now_num = raw_json["item"]["throttling"]["numUserMessagesInConversation"]
        reply = raw_json["item"]["messages"][1]["text"]
        reply = re.sub(r"\[\^(\d+)\^\]", r"[\1]", reply)
        reply = re.sub(r"\[?\^(\d+)\^\]?", r"[\1]", reply)
    except:
        current_userdata.is_waiting = False
        await matcher.finish(reply_out(event, "出错喽，多次重试都出错的话，联系机器人主人"))
    msg_text = MessageSegment.text(reply + "  \n")
    if newbing_persistor.suggest_able == "True":
        try:
            suggests = [
                raw_json["item"]["messages"][1]["suggestedResponses"][i]["text"]
                for i in range(
                    len(raw_json["item"]["messages"][1]["suggestedResponses"])
                )
            ]
            suggest_str = "\n".join(
                [f"{i+1}. {suggestion}  " for i, suggestion in enumerate(suggests)]
            )
            msg_text += MessageSegment.text("  \n建议回复:  \n" + suggest_str)
        except:
            pass
    else:
        suggests = []
    forward_rawmsg = None
    forward_msg = None

    # 尝试解析图片或其他资源
    try:
        image_sources = [
            a["seeMoreUrl"]
            for a in raw_json["item"]["messages"][1]["sourceAttributions"]
            if "seeMoreUrl" in a
        ]
        image_names = [
            a["providerDisplayName"]
            for a in raw_json["item"]["messages"][1]["sourceAttributions"]
            if "providerDisplayName" in a
        ]
        image_links = [
            a["imageLink"]
            for a in raw_json["item"]["messages"][1]["sourceAttributions"]
            if "imageLink" in a
        ]
        for i in range(len(image_sources)):
            source_link = image_sources[i]
            display_name = image_names[i]
            each_msg = MessageSegment.text(f"[{i+1}] {display_name}:\n{source_link}\n")
            if len(image_links) - i:
                image_link = image_links[i]
                each_msg += MessageSegment.image(image_link)
            forward_msg += MessageSegment.node_custom(
                user_id=current_userdata.sender.user_id,
                nickname=current_userdata.sender.user_name,
                content=each_msg,
            )
            forward_rawmsg += each_msg
    except:
        pass

    msg_text += MessageSegment.text(f"\n回复上限:{now_num}/{max_num}  ")

    current_userdata.is_waiting = False

    reply_msgid_container = await sendmsg(str(msg_text), matcher, event)

    current_userdata.last_reply_message_id = reply_msgid_container["message_id"]
    current_userdata.last_suggests = suggests
    if forward_msg or forward_rawmsg:
        is_send = False
        try:
            # 尝试合并转发
            if forward_msg and event.message_type == "group":
                reply_msgid_container = await bot.send_group_forward_msg(
                    group_id=event.group_id, messages=forward_msg
                )
            elif forward_msg and event.message_type == "private":
                reply_msgid_container = await bot.send_private_forward_msg(
                    user_id=event.user_id, messages=forward_msg
                )
            current_userdata.last_reply_message_id = reply_msgid_container["message_id"]
            is_send = True
        except:
            if not is_send:
                try:
                    reply_msgid_container = await matcher.send(
                        reply_out(event, forward_rawmsg)
                    )
                    current_userdata.last_reply_message_id = reply_msgid_container[
                        "message_id"
                    ]
                except:
                    pass
            else:
                pass

    if now_num == max_num:
        await matcher.send(reply_out(event, "已达到单次对话上限，自动为您刷新对话"))
        try:
            chatbot = await Chatbot.create(cookie_path=newbing_persistor.cookie_path_)
            current_userdata.chatbot = chatbot
        except Exception as e:
            await matcher.finish(reply_out(event, f"自动刷新出错:{e}"))

    await matcher.finish()


newbing_change_mode = on_command(
    "bc", aliases={"bingchange", "bing切换"}, priority=1, block=False
)

@newbing_change_mode.handle()
async def __newbing_change_mode__(
    matcher: Matcher,
    state: T_State,
    event: MessageEvent,
    bot: Bot,
    args: Message = CommandArg(),
):
    try:
        current_userinfo, current_userdata = set_userdata(
            event=event, user_data_dict=user_data_dict
        )
    except:
        await matcher.finish()
    if args and len(args[0]) == 1 and args[0] in ["1", "2", "3"]:
        current_userdata.chatmode = args[0]
        if args[0] == "1":
            style = "creative 创造力模式"
        elif args[0] == "2":
            style = "balanced 均衡模式"
        else:
            style = "precise 精确模式"
        await matcher.finish(reply_out(event, f"已切换为: {style}"))
    msg = "请输入模式前索引数字\n1:creative 创造力模式\n2:balanced 均衡模式\n3:precise 精确模式"
    reply = await matcher.send(reply_out(event, msg))
    state["reply"] = reply
    state["current_userdata"] = current_userdata


@newbing_change_mode.got("mode")
async def newbing_change_mode__(
    matcher: Matcher,
    state: T_State,
    event: MessageEvent,
    bot: Bot,
    infos: str = ArgStr("mode"),
):
    current_userdata = state["current_userdata"]
    reply = state["reply"]
    mode = infos[0]
    if mode and len(mode) == 1 and mode in ["1", "2", "3"]:
        current_userdata.chatmode = mode
        if mode == "1":
            style = "creative 创造力模式"
        elif mode == "2":
            style = "balanced 均衡模式"
        else:
            style = "precise 精确模式"
        await matcher.send(reply_out(event, f"已切换为: {style}"))
        await bot.delete_msg(message_id=reply["message_id"])
        await matcher.finish()
    else:
        await matcher.send(reply_out(event, f"你输入的数字有误，请重新输入"))
        await matcher.reject()


bing_draw = on_command(
    "bingdraw", aliases={"bd", "bing绘图", "bing画画"}, priority=1, block=False
)


@bing_draw.handle()
async def __newbing_change_mode__(
    matcher: Matcher,
    state: T_State,
    event: MessageEvent,
    bot: Bot,
    args: Message = CommandArg(),
):
    try:
        current_userinfo, current_userdata = set_userdata(
            event=event, user_data_dict=user_data_dict
        )
    except:
        await matcher.finish()
    if not args[0]:
        await matcher.finish()
    else:
        prompt = str(args[0])

    if current_userdata.is_waiting:
        await matcher.finish(reply_out(event, f"你有一个对话进行中，请等结束后再继续询问"))
    try:
        with open(newbing_persistor.cookie_path_, encoding="utf-8") as file:
            cookie_json = json.load(file)
            for cookie in cookie_json:
                if cookie.get("name") == "_U":
                    auth_cookie = cookie.get("value")
                    break
            else:
                logger.warning("newbing的cookie内容有误,无法使用，跳过")
                await matcher.finish()
    except FileNotFoundError:
        logger.warning("newbing cookie未配置,无法使用，跳过")
        await matcher.finish()
    current_userdata.is_waiting = True
    try:
        async with ImageGenAsync(auth_cookie) as image_generator:
            image_links = await image_generator.get_images(prompt)
            max_retry = 3  # 最大重试次数
            retry_count = 0  # 当前重试次数
            while retry_count < max_retry:
                try:
                    temp_path = "./data/spark_gpt/temp/"
                    jpeg_files = list(Path(temp_path).glob("*.jpeg"))
                    # 删除所有JPEG格式的文件
                    for file_path in jpeg_files:
                        file_path.unlink()

                    await image_generator.save_images(image_links, temp_path)
                    break  # 如果保存成功，则跳出循环
                except Exception as e:
                    retry_count += 1
                    print(f"Error occurred while saving images: {e}")
                    if retry_count == max_retry:
                        # 如果达到最大重试次数还是保存失败，则抛出异常
                        await matcher.finish(reply_out(event, f"下载图片出错，多次出错请联系机器人主人"))
    except:
        current_userdata.is_waiting = False
        await matcher.finish(reply_out(event, f"生成图片出错，多次出错请联系机器人主人"))

    current_userdata.is_waiting = False

    try:
        forward_msg = MessageSegment.node_custom(
            user_id=current_userdata.sender.user_id,
            nickname=current_userdata.sender.user_name,
            content=MessageSegment.text("绘图结果如下:\n"),
        )
        image_msg = MessageSegment.text("绘图结果如下:\n")
        for i in range(4):
            image_msg += MessageSegment.image(Path(temp_path + f"{i}.jpeg"))
            forward_msg += MessageSegment.node_custom(
                user_id=current_userdata.sender.user_id,
                nickname=current_userdata.sender.user_name,
                content=MessageSegment.image(Path(temp_path + f"{i}.jpeg")),
            )

    except:
        await matcher.finish(reply_out(event, "发送失败，多次失败请联系机器人主人"))
    try:
        # 尝试合并转发
        if forward_msg and event.message_type == "group":
            await bot.send_group_forward_msg(
                group_id=event.group_id, messages=forward_msg
            )
        elif forward_msg and event.message_type == "private":
            await bot.send_private_forward_msg(
                user_id=event.user_id, messages=forward_msg
            )
    except:
        await matcher.finish(reply_out(event, image_msg))


######################################################
newbing_help = on_command("binghelp", aliases={"bing帮助", "bh"}, priority=4, block=False)


@newbing_help.handle()
async def __newbing_help__(matcher: Matcher):
    if not is_useable:
        await matcher.finish()
    msg = """
# spark-gpt NewBing使用说明

- 机器人对每个人都是相互独立的。

- !!! 以下命令前面全部要加 '/' !!!  

## 对话命令
- 支持以下特性：可以通过对话 (清除/清空)(对话/历史)或"刷新对话"或"清空历史对话"来开启另一个新对话  
- 对话一次后，就可以直接回复机器人给你的最后的回复来进行连续对话  
- 可以通过建议回复的数字索引来使用建议回复  

| 命令 | 描述 |
| --- | --- |
| `/bing / b + 内容` | 对话功能|

## 切换对话模式命令

| 命令 | 描述 |
| --- | --- |
| `/bc / bingchange / bing切换 (+ 数字)` | 切换对话模式，支持以下三种模式：1. 创造力模式；2. 均衡模式；3. 精确模式。 |

## 画图命令

| 命令 | 描述 |
| --- | --- |
| `/bingdraw / bd / bing绘图(bing画图) + 要画的东西(中文/英文)` | Dall-e画图功能，可以画出指定的中文或英文内容。 |"""
    pic = await md_to_pic(msg)
    await matcher.send(MessageSegment.image(pic))
    await matcher.finish()



import json
from typing import Dict, List, Optional, Tuple
import nonebot
from nonebot.adapters.onebot.v11 import MessageEvent
from pathlib import Path
from pydantic import BaseModel, Extra
import json


class BotInfo(BaseModel):
    # BotInfo本身储存就决定了它是单平台单来源的
    # 这里预留一些东西只是留作可能的修改使用
    nickname: Optional[str]
    truename: Optional[str]
    source: Optional[str]
    model: Optional[str]
    num_users: Optional[int]
    prompt_nickname: Optional[str]
    prompt: Optional[str]
    # 这里做owner主要是用来hash，以区分不同用户的可能重名的bot
    owner: Optional[str]

    last_suggests: Optional[List[str]] = []

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.nickname, self.truename, self.owner))

    @classmethod
    def from_dict(cls, data: dict) -> "BotInfo":
        return cls(**data)


class UserInfo(BaseModel):
    platform: str
    user_id: int

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.platform, self.user_id))

    @classmethod
    def from_dict(cls, data: dict) -> "UserInfo":
        return cls(**data)

    def to_string(self) -> str:
        return f"{self.platform}-{self.user_id}"

    @classmethod
    def from_string(cls, s: str) -> "UserInfo":
        platform, user_id = s.split("-")
        return cls(platform=platform, user_id=int(user_id))


class Sender(BaseModel, extra=Extra.ignore):
    user_id: int
    user_name: str

    def __hash__(self):
        return hash((self.user_id, self.user_name))

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "Sender":
        return cls(**data)


class UserData(BaseModel):
    sender: Sender
    wait_msg_id: Optional[int] = None
    last_reply_message_id: Dict[str, int] = {}
    is_waiting: bool = False

    def __hash__(self):
        return hash(
            (
                self.sender,
                self.wait_msg_id,
                self.last_reply_message_id,
                self.last_reply_message_id,
            )
        )

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "UserData":
        return cls(**data)


def get_user_info_and_data(event: MessageEvent) -> Tuple[UserInfo, UserData]:
    """
    接受一个 event ，返回一个包含 UserInfo 和 UserData 的元组。
    仅仅将构造好的 UserInfo 和 UserData 构建成一个一个元组并返回。
    用来初次生成UserInfo和UserData
    """
    return (
        UserInfo(platform="qq", user_id=event.user_id),
        UserData(
            sender=Sender(
                user_id=event.user_id,
                user_name=event.sender.nickname or "<未知的的用户名>",
            )
        ),
    )


def set_userdata(
    event: MessageEvent, user_data_dict: dict[UserInfo, UserData]
) -> UserData:
    """
    接受一个event和一个user_data_dict(全局的，所有用户的),在user_data_dict匹配(无则初次设置)用户的过往使用信息
    返回UserInfo和匹配后的(无则新建的)对应用户的current_user_dict结构为[UserInfo, UserData]
    """
    user_info = UserInfo(platform="qq", user_id=event.user_id)
    user_data = UserData(
        sender=Sender(
            user_id=event.user_id,
            user_name=event.sender.nickname or "<未知的的用户名>",
        )
    )
    return user_info, user_data_dict.setdefault(user_info, user_data)


# 储存spark的持久数据,提供一键保存和加载功能
class SparkPersistor(BaseModel):
    path_: str = str(Path()) + "/data/spark_gpt/spark_data.json"
    pic_able: str = None
    url_able: str = "True"
    suggest_able: str = "True"
    num_limit: int = 350
    superusers: List[str] = []
    blacklist: List[str] = []
    whitelist: List[str] = []
    mode: str = "black"
    prompts_dict: Dict[str, str] = {
        "猫娘": "现在你将模仿一只猫娘，与我对话每一句话后面都要加上“喵”，如果你能明白我的意思，请回复“喵~你好主人”",
        "默认": "一个ai语言模型",
    }

    def __init__(self, **data):
        super().__init__(**data)
        try:
            saved_data = self.load()
            self.__dict__.update(saved_data)
        except:
            pass
        get_config = nonebot.get_driver().config
        self.superusers = (
            get_config.spark_superusers
            if hasattr(get_config, "spark_superusers")
            else self.superusers
        )
        self.blacklist = (
            get_config.spark_blacklist
            if hasattr(get_config, "spark_blacklist")
            else self.blacklist
        )
        self.whitelist = (
            get_config.spark_whitelist
            if hasattr(get_config, "spark_whitelist")
            else self.whitelist
        )
        self.mode = (
            get_config.spark_mode
            if hasattr(get_config, "spark_mode")
            and get_config.spark_mode in ["white", "black"]
            else self.mode
        )
        self.pic_able = getattr(get_config, "spark_picable", None)
        self.url_able = getattr(get_config, "spark_urlable", "True")
        self.suggest_able = getattr(get_config, "spark_suggestable", "True")
        self.num_limit = int(getattr(get_config, "spark_limit", 350))
        self.save()

    def save(self):
        Path(self.path_).parent.mkdir(parents=True, exist_ok=True)
        data = self.__dict__.copy()
        with open(self.path_, "w", encoding="utf-8") as f:
            json.dump(data, f, ensure_ascii=False)

    def load(self):
        with open(self.path_, "r", encoding="utf-8") as f:
            data = json.load(f)
        return data


# 这里的配置即使在多平台，也应只初始化一次
spark_persistor = SparkPersistor()



import asyncio
from nonebot.plugin import on_command, on_message
from nonebot.params import ArgStr, CommandArg
from nonebot.typing import T_State
from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import (
    Message,
    Event,
    Bot,
    MessageEvent,
    MessageSegment,
)
from .config import (
    BotInfo,
    PoeTemper,
    poe_persistor,
    get_user_info_and_data,
    set_userdata,
)
from .poe_func import (
    generate_truename,
    reply_out,
    delete_messages,
    is_useable,
    send_msg,
    close_page,
    is_vip,
)
from .poe_api import poe_chat, poe_create, poe_clear
from ..common.config import spark_persistor
from ..common.render.render import md_to_pic
from ..chatgpt_web.config import gptweb_persistor
from .pwframework import pwfw
from nonebot import logger

# 初始化两个需要使用的实例
temp_data = PoeTemper()
user_data_dict = temp_data.user_data_dict
msg_bot_bidict = temp_data.msg_bot_bidict
base_botinfo_dict = temp_data.base_botinfo_dict
prompts_dict = spark_persistor.prompts_dict
logger.info("开始加载poe")
######################################################
poe_auto_change_prompt = on_command(
    "poechangeprompt", aliases={"pcp"}, priority=4, block=False
)


@poe_auto_change_prompt.handle()
async def __poe_auto_change_prompt__(event: Event):
    global poe_persistor
    user_id = str(event.user_id)
    if user_id not in poe_persistor.superusers:
        await poe_auto_change_prompt.finish("你不是管理员哦")
    now_prompt = poe_persistor.auto_prompt
    str_prompts = str()
    i = 1
    for key, value in prompts_dict.items():
        str_prompts += f"*******************\n{i}:预设名称：{key}\n预设内容：{value}\n"
        i += 1
    await poe_auto_change_prompt.send(
        f"现在的自动创建预设是{now_prompt}\n当前可用预设有：\n{str_prompts}"
    )


@poe_auto_change_prompt.got("name", prompt="请输入要切换到的预设名称\n输入取消 或 算了可以终止创建")
async def __poe_auto_change_prompt____(
    matcher: Matcher, event: Event, state: T_State, infos: str = ArgStr("name")
):
    if not is_useable(event):
        await matcher.finish()
    if infos in ["取消", "算了"]:
        await poe_auto_change_prompt.finish("终止切换")
    infos = infos.split(" ")
    if len(infos) != 1 or infos[0] not in prompts_dict:
        await poe_auto_change_prompt.reject("你输入的信息有误，请检查后重新输入")
    # 将更新后的字典写回到JSON文件中
    poe_persistor.auto_prompt = infos[0]
    poe_persistor.save()
    await poe_auto_change_prompt.finish("成功切换默认自动创建prompt")


######################################################
creat_lock = asyncio.Lock()
poe_create_ = on_command("poecreate", aliases={"pc"}, priority=4, block=False)


@poe_create_.handle()
async def __(matcher: Matcher, state: T_State, event: Event):
    global poe_persistor
    if not is_useable(event):
        await matcher.finish()
    create_msgs = []
    if len(spark_persistor.prompts_dict) > 0:
        str_prompts = str()
        for key, _ in spark_persistor.prompts_dict.items():
            str_prompts += f"{key}\n"
        # create_msgs.append(await matcher.send(reply_out(event, f"当前预设有：\n{str_prompts}")))
        msg = f'当前预设有：\n{str_prompts}\n请输入\n1.机器人名称,\n2.基础模型选项，可选项为（gpt3_5输入1,claude输入2）,\n3.自定义预设（预设内容中间不要有空格） 或 "." + 可用本地预设名\n三个参数中间用空格隔开\n最终格式示例:\n示例一：chat 2 一个智能助理\n示例二： chat 1 .默认\n输入取消 或 算了可以终止创建'
    else:
        msg = f'当前没有可用本地预设\n\n请输入\n1.机器人名称,\n2.基础模型选项，可选项为（gpt3_5输入1,claude输入2）,\n3.自定义预设（预设内容中间不要有空格） 或 "." + 可用本地预设名\n三个参数中间用空格隔开\n最终格式示例:\n示例一：chat 2 一个智能助理\n示例二： chat 1 .默认\n输入取消 或 算了可以终止创建'
    create_msgs.append(await matcher.send(reply_out(event, msg)))
    state["create_msgs"] = create_msgs


@poe_create_.got("model")
async def __poe_create___(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    infos: str = ArgStr("model"),
):
    global poe_persistor
    create_msgs = state["create_msgs"]
    user_id = str(event.user_id)
    if infos in ["取消", "算了"]:
        create_msgs.append(await matcher.send(reply_out(event, "取消创建")))
        await delete_messages(bot, str(event.user_id), create_msgs)
        await poe_create_.finish()

    infos = infos.split(" ", 2)
    if not (len(infos) == 3 and infos[1] in ["1", "2"]):
        create_msgs.append(await matcher.send(reply_out(event, "输入信息有误，请检查后重新输入")))
        await poe_create_.reject()

    # 获取创建所需信息

    nickname = str(infos[0])

    truename = str(generate_truename(user_id, nickname))
    prompt = str(infos[2])
    bot_index = str(infos[1])
    if bot_index == "1":
        model = "gpt3.5"
    else:
        model = "claude-instant"
    prompt_name = str("unknown")
    if prompt.startswith("."):
        prompt_name = prompt[1:]
        if prompt_name in spark_persistor.prompts_dict:
            prompt = spark_persistor.prompts_dict[prompt_name]
        else:
            create_msgs.append(
                await matcher.send(reply_out(event, "输入的本地预设名不正确，请重新输入"))
            )
            await poe_create_.reject()

    current_userinfo, current_userdata = set_userdata(event, user_data_dict)
    if current_userinfo not in list(poe_persistor.user_dict.keys()):
        poe_persistor.user_dict.setdefault(current_userinfo, {"all": {}, "now": {}})

    # 查看对应用户下是不是有重名的bot
    if (
        current_userinfo in gptweb_persistor.user_dict
        and nickname in gptweb_persistor.user_dict[current_userinfo]["all"]
    ) or (
        current_userinfo in poe_persistor.user_dict
        and nickname in poe_persistor.user_dict[current_userinfo]["all"]
    ):
        create_msgs.append(
            await matcher.send(reply_out(event, "已经有同名的bot了，换一个名字重新输入吧"))
        )
        await matcher.reject()
    if creat_lock.locked():
        waitmsg = await matcher.send(reply_out(event, "有人正在创建中，稍后自动为你创建"))
    async with creat_lock:
        page = await pwfw.new_page()
        is_created = await poe_create(
            page=page, botname=truename, base_bot_index=int(bot_index), prompt=prompt
        )
        await page.close()
        # is_created = True
        if is_created:
            # # 将更新后的字典写回到JSON文件中
            botinfo = BotInfo(
                nickname=nickname,
                truename=truename,
                source="poe",
                model=model,
                prompt_nickname=prompt_name,
                prompt=prompt,
                owner=str(event.user_id),
            )
            poe_persistor.user_dict.setdefault(current_userinfo, {}).setdefault(
                "all", {}
            )[nickname] = botinfo

            poe_persistor.user_dict[current_userinfo]["now"] = {nickname: botinfo}

            poe_persistor.save()
            try:
                await bot.delete_msg(message_id=waitmsg["message_id"])
            except:
                pass
            await matcher.send(reply_out(event, "创建成功并切换到新建bot"))

            await delete_messages(bot, user_id, create_msgs)
            await matcher.finish()
        else:
            await matcher.send(reply_out(event, "出错了，多次出错请联系机器人管理员"))

            await delete_messages(bot, user_id, create_msgs)
            await matcher.finish()


# ######################################################
poe_remove = on_command("poeremove", aliases={"pr"}, priority=4, block=False)


@poe_remove.handle()
async def __poe_remove__(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    args: Message = CommandArg(),
):
    if not is_useable(event):
        await matcher.finish()
    userinfo, _ = get_user_info_and_data(event)

    remove_msgs = []
    if userinfo not in list(poe_persistor.user_dict.keys()):
        await matcher.finish(reply_out(event, "你还没创建任何bot"))

    # 如果有直接的arg的nickname，直接删除它，否则跳过
    if str(args):
        nickname = str(args[0])
        if nickname not in poe_persistor.user_dict[userinfo]["all"]:
            await matcher.finish(reply_out(event, "没有这个机器人呢"))
        if nickname == str(list(poe_persistor.user_dict[userinfo]["now"].keys())[0]):
            await matcher.finish(reply_out(event, "不能删除正在使用的bot哦"))
        del poe_persistor.user_dict[userinfo]["all"][nickname]
        msg = f"已删除{nickname}"
        await matcher.finish(reply_out(event, msg))

    bots = list(poe_persistor.user_dict[userinfo]["all"].keys())
    if len(bots) == 1:
        await matcher.finish(reply_out(event, f"当前只有一个机器人:{bots[0]},不能删除"))
    bot_nickname_str = "\n".join(str(bot) for bot in bots)

    nickname = str(list(poe_persistor.user_dict[userinfo]["now"].keys())[0])
    msg = (
        "你已经创建的的bot有：\n"
        + bot_nickname_str
        + f"\n当前使用的bot是{nickname}\n\n请输入要删除的机器人名称\n输入取消 或 算了可以终止创建"
    )
    remove_msgs.append(await matcher.send(reply_out(event, msg)))
    state["remove_msgs"] = remove_msgs
    state["userinfo"] = userinfo


@poe_remove.got("nickname")
async def __poe_remove____(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    infos: str = ArgStr("nickname"),
):
    userinfo = state["userinfo"]
    remove_msgs = state["remove_msgs"]
    bots = list(poe_persistor.user_dict[userinfo]["all"].keys())
    if infos in ["取消", "算了"]:
        remove_msgs.append(await matcher.send(reply_out(event, "终止删除")))

        await delete_messages(bot, str(event.user_id), remove_msgs)
        await matcher.finish()
    infos = infos.split(" ")
    nickname_delete = infos[0]
    nickname_now = str(list(poe_persistor.user_dict[userinfo]["now"].keys())[0])
    if not (nickname_delete in bots):
        remove_msgs.append(await matcher.send(reply_out(event, "输入信息有误，请检查后重新输入")))
        await poe_remove.reject()
    if nickname_delete == nickname_now:
        remove_msgs.append(await matcher.send(reply_out(event, "不能删除正在使用的bot哦")))

        await delete_messages(bot, userinfo, remove_msgs)
        await poe_remove.finish()
    del poe_persistor.user_dict[userinfo]["all"][nickname_delete]
    poe_persistor.save()

    await matcher.send(reply_out(event, f"已删除{nickname_delete}"))
    await delete_messages(bot, userinfo, remove_msgs)
    await matcher.finish()


#############################################################
poe_switch = on_command("poeswitch", aliases={"ps"}, priority=4, block=False)


@poe_switch.handle()
async def __poe_switch__(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    args: Message = CommandArg(),
):
    if not is_useable(event):
        await matcher.finish()
    if str(event.message).startswith(
        ("/psg", "/psn", "/psd", "/pss", "/psc+", "/psg4", "/psc")
    ):
        await matcher.finish()
    userinfo, _ = get_user_info_and_data(event)

    switch_msgs = []
    if userinfo not in list(poe_persistor.user_dict.keys()):
        await matcher.finish(reply_out(event, "你还没创建任何bot"))
    if args:
        nickname = str(args[0])
        if nickname not in poe_persistor.user_dict[userinfo]["all"].keys():
            switch_msgs.append(await matcher.finish(reply_out(event, "没有这个机器人呢")))

        poe_persistor.user_dict[userinfo]["now"] = {
            nickname: poe_persistor.user_dict[userinfo]["all"][nickname]
        }
        poe_persistor.save()
        await matcher.send(reply_out(event, f"已切换为{nickname}"))
        await delete_messages(bot, userinfo, switch_msgs)
        await poe_switch.finish()

    bots = list(poe_persistor.user_dict[userinfo]["all"].keys())
    bot_str = "\n".join(str(bot) for bot in bots)

    nickname = str(list(poe_persistor.user_dict[userinfo]["now"].keys())[0])
    if len(bots) == 1:
        await matcher.finish(reply_out(event, f"当前只有一个bot:{nickname},无法切换"))
    msg = (
        "你已经创建的的bot有：\n"
        + bot_str
        + f"\n当前使用的bot是{nickname}\n\n请输入要切换的机器人名称\n输入取消 或 算了可以终止创建"
    )
    switch_msgs.append(await matcher.send(reply_out(event, msg)))

    state["switch_msgs"] = switch_msgs
    state["userinfo"] = userinfo


@poe_switch.got("nickname")
async def __poe_switch____(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    infos: str = ArgStr("nickname"),
):
    switch_msgs = state["switch_msgs"]
    userinfo = state["userinfo"]

    if infos in ["取消", "算了"]:
        switch_msgs.append(await matcher.send(reply_out(event, "中断切换")))
        await delete_messages(bot, userinfo, switch_msgs)
        await matcher.finish()

    nickname = infos.split(" ")[0]
    if nickname not in list(poe_persistor.user_dict[userinfo]["all"].keys()):
        switch_msgs.append(await matcher.send(reply_out(event, "没有这个机器人呢")))
        await poe_switch.reject()

    poe_persistor.user_dict[userinfo]["now"] = {
        nickname: poe_persistor.user_dict[userinfo]["all"][nickname]
    }
    poe_persistor.save()
    await matcher.send(reply_out(event, f"已切换为{nickname}"))
    await delete_messages(bot, userinfo, switch_msgs)
    await poe_switch.finish()


#############################################################
async def _is_chat_(event: MessageEvent, bot: Bot):
    # 处理用命令来进行的对话,包括pt，指定名，公共的，也可以处理回复形式的对话
    try:
        current_userinfo, current_userdata = set_userdata(
            event=event, user_data_dict=user_data_dict
        )
    except:
        return False
    if bool(event.reply):
        if str(event.reply.sender.user_id) == bot.self_id:
            raw_message = str(event.message)
            if not raw_message:
                return False
            try:
                botinfo = msg_bot_bidict[event.reply.message_id]
                if not is_useable(event):
                    return False
                if botinfo in list(base_botinfo_dict.values()):
                    mode = "public"
                elif botinfo.nickname in current_userdata.last_reply_message_id:
                    mode = "private"
                else:
                    return False
                try:
                    last_suggests = msg_bot_bidict[
                        event.reply.message_id
                    ].last_suggests
                except:
                    last_suggests = []
                    #  mode,raw_message,last_msgid,last_suggests,nickname,truename,current_userinfo,current_userdata = temp
                return (
                    mode,
                    raw_message,
                    event.reply.message_id,
                    last_suggests,
                    botinfo.nickname,
                    botinfo.truename,
                    current_userinfo,
                    current_userdata,
                )
            except:
                return False
        else:
            return False
    elif str(event.message).startswith(("/pt", "/ptalk")):
        if not is_useable(event):
            return False
        raw_message = (
            str(event.message)
            .replace("/pchat ", "")
            .replace("/pchat", "")
            .replace("/pt ", "")
            .replace("/pt", "")
        )
        if not raw_message:
            return False
        mode = "private"
        try:
            botinfo = list(
                poe_persistor.user_dict[current_userinfo]["now"].values()
            )[0]
            nickname = botinfo.nickname
            truename = botinfo.truename
        except:
            return "none"
        try:
            last_msgid = current_userdata.last_reply_message_id[nickname]
            last_suggests = msg_bot_bidict[last_msgid].last_suggests
        except:
            last_msgid = 0
            last_suggests = []
        return (
            mode,
            raw_message,
            last_msgid,
            last_suggests,
            nickname,
            truename,
            current_userinfo,
            current_userdata,
        )
    else:
        try:
            bots_nicknames = list(
                poe_persistor.user_dict[current_userinfo]["all"].keys()
            )
            nickname = next(
                (
                    name
                    for name in bots_nicknames
                    if str(event.message).startswith("/" + name)
                ),
                None,
            )
            mode = "private"
            if nickname:
                if not is_useable(event):
                    return False
                truename = poe_persistor.user_dict[current_userinfo]["all"][
                    nickname
                ].truename
                raw_message = (
                    str(event.message)
                    .replace("/" + nickname + " ", "")
                    .replace("/" + nickname, "")
                )
                if not raw_message:
                    return False
                try:
                    last_msgid = current_userdata.last_reply_message_id[nickname]
                    last_suggests = msg_bot_bidict[last_msgid].last_suggests
                except:
                    last_msgid = 0
                    last_suggests = []
                return (
                    mode,
                    raw_message,
                    last_msgid,
                    last_suggests,
                    nickname,
                    truename,
                    current_userinfo,
                    current_userdata,
                )
            else:
                bots_nicknames = list(base_botinfo_dict.keys())
                nickname = next(
                    (
                        name
                        for name in base_botinfo_dict
                        if str(event.message).startswith("/" + name)
                    ),
                    None,
                )
                if nickname:
                    if not is_useable(event):
                        return False
                    truename = base_botinfo_dict[nickname].truename
                    raw_message = (
                        str(event.message)
                        .replace("/" + nickname + " ", "")
                        .replace("/" + nickname, "")
                    )
                    if nickname in ["psg4", "psc+"] and not is_vip(event):
                        return False
                    if not raw_message:
                        return False
                    mode = "public"
                    try:
                        last_msgid = current_userdata.last_reply_message_id[
                            nickname
                        ]
                        last_suggests = msg_bot_bidict[last_msgid].last_suggests
                    except:
                        last_msgid = 0
                        last_suggests = []
                    return (
                        mode,
                        raw_message,
                        last_msgid,
                        last_suggests,
                        nickname,
                        truename,
                        current_userinfo,
                        current_userdata,
                    )
                else:
                    return False
        except:
            return False

#############################################################
chat_lock = asyncio.Semaphore(3)
poe_base_lock = asyncio.Lock()
poe_chat_ = on_message(priority=1, block=False)


@poe_chat_.handle()
async def __chat_bot__(matcher: Matcher, event: MessageEvent, bot: Bot):
    temp = await _is_chat_(event, bot)
    if temp == False:
        await matcher.finish()
    if temp == "none":
        mode = "private"
        user_id = str(event.user_id)
        raw_message = (
            str(event.message)
            .replace("/pchat ", "")
            .replace("/pchat", "")
            .replace("/pt ", "")
            .replace("/pt", "")
        )
        last_suggests = []
        nickname = "pdefault"
        truename = str(generate_truename(user_id, nickname))
        prompt_nickname = poe_persistor.auto_prompt
        prompt = spark_persistor.prompts_dict[prompt_nickname]
        page = await pwfw.new_page()
        is_created = await poe_create(page, truename, 1, prompt)
        await page.close()
        current_userinfo, current_userdata = set_userdata(
            event=event, user_data_dict=user_data_dict
        )
        if is_created:
            # # 将更新后的字典写回到JSON文件中
            botinfo = BotInfo(
                nickname=nickname,
                truename=truename,
                source="poe",
                model="gpt3.5",
                prompt_nickname=prompt_nickname,
                prompt=prompt,
                owner="qq" + user_id,
            )
            poe_persistor.user_dict.setdefault(current_userinfo, {}).setdefault(
                "all", {}
            )["pdefault"] = botinfo
            poe_persistor.user_dict[current_userinfo]["now"] = {nickname: botinfo}

            poe_persistor.save()
            await matcher.send(reply_out(event, "自动创建gpt3.5机器人:pdefault成功"))
        else:
            await matcher.finish(reply_out(event, "自动创建出错，多次出错请联系机器人管理员"))
    else:
        (
            mode,
            raw_message,
            last_msgid,
            last_suggests,
            nickname,
            truename,
            current_userinfo,
            current_userdata,
        ) = temp
        try:
            botinfo = msg_bot_bidict[last_msgid]
        except:
            if mode == "private":
                botinfo = poe_persistor.user_dict[current_userinfo]["all"][nickname]
            else:
                botinfo = base_botinfo_dict[nickname]
    if len(raw_message) == 1 and raw_message in ["1", "2", "3", "4"] and last_suggests:
        text = last_suggests[int(raw_message) - 1]
    else:
        text = raw_message

    if mode == "private":
        if current_userdata.is_waiting:
            await matcher.finish(reply_out(event, "你已经有一个对话进行中了，请等结束后再发送"))
        if chat_lock.locked():
            await matcher.send(reply_out(event, "请稍等,你前面已有3个用户,你的回答稍后就来"))

        async with chat_lock:
            if text in ["清除对话", "清空对话", "清除历史", "清空历史", "清除记录", "清空历史对话", "刷新对话"]:
                current_userdata.is_waiting = True
                page = await pwfw.new_page()
                is_cleared = await poe_clear(page=page, truename=truename)
                await page.close()
                if is_cleared:
                    msg = f"成功清除了{nickname}的历史消息"
                else:
                    msg = "出错了，多次错误请联系机器人主人"
                current_userdata.is_waiting = False
                reply_msgid = await matcher.send(reply_out(event, msg))
                current_userdata.last_reply_message_id[nickname] = reply_msgid[
                    "message_id"
                ]
                msg_bot_bidict.inv[botinfo] = reply_msgid["message_id"]
                await matcher.finish()
            page = await pwfw.new_page()
            current_userdata.is_waiting = True
            result = await poe_chat(truename, text, page)
            await page.close()
            current_userdata.is_waiting = False

            reply_msgid, chat_suggest_temp = await send_msg(result, matcher, event)
            current_userdata.last_reply_message_id[nickname] = reply_msgid["message_id"]
            if len(chat_suggest_temp) - 0:
                botinfo.last_suggests = chat_suggest_temp
            msg_bot_bidict.inv[botinfo] = reply_msgid["message_id"]
            await matcher.finish()
    elif mode == "public":
        if poe_base_lock.locked() and botinfo.num_users <= 4:
            base_botinfo_dict[nickname].num_users += 1
            waitmsg = await matcher.send(reply_out(event, "还没回答完上一个问题，稍等马上回复你"))
        elif botinfo.num_users > 4:
            await matcher.finish(reply_out(event, "我还有5个问题没回答呢，你等会再用吧"))

        async with poe_base_lock:
            if text in ["清除对话", "清空对话", "清除历史", "清空历史", "清空历史对话", "刷新对话"]:
                current_userdata.is_waiting = True
                page = await pwfw.new_page()
                is_cleared = await poe_clear(page=page, truename=truename)
                await page.close()
                if is_cleared:
                    msg = f"成功清除了{truename}的历史消息"
                else:
                    msg = "出错了，多次错误请联系机器人主人"
                current_userdata.is_waiting = False
                reply_msgid = await matcher.send(reply_out(event, msg))
                current_userdata.last_reply_message_id[nickname] = reply_msgid[
                    "message_id"
                ]
                msg_bot_bidict.inv[botinfo] = reply_msgid["message_id"]
                await matcher.finish()
            page = await pwfw.new_page()
            result = await poe_chat(truename, text, page)
            await close_page(page)
            msgid, temp_suggests = await send_msg(result, matcher, event)
            base_botinfo_dict[nickname].num_users -= 1
            if len(temp_suggests) > 0:
                botinfo.last_suggests = temp_suggests

            msg_bot_bidict.inv[botinfo] = msgid["message_id"]
            try:
                await bot.delete_msg(message_id=waitmsg["message_id"])
            except:
                pass
            await matcher.finish()


######################################################
poe_help = on_command("poehelp", aliases={"poe帮助", "ph"}, priority=4, block=False)


@poe_help.handle()
async def __poe_help__(bot: Bot, matcher: Matcher, event: Event):
    user_id = str(event.user_id)
    if not is_useable(event):
        await matcher.finish()
    msg = """
# spark-gpt Poe使用说明

- 共享的机器人供多人共同使用，而用户隔离的机器人每个人都是相互独立的。

- 以下命令前面全部要加 '/' ！！！！！

- 对话问答功能均支持以下特性：

- 可以通过回复机器人的最后一个回答来继续对话，而无需命令。
- 可以回复 "(清除/清空)(对话/历史)" 或 刷新对话, 清除对话历史 或 "pd", "poedump", "pdump" 来清空对话。
- 可以通过建议回复的数字索引来使用建议回复。

************************

- 以下命令均支持用户隔离

| 命令 | 描述 |
| --- | --- |
| `/ptalk / pt + 你要询问的内容` | 对话功能，如果没创建机器人，对话将自动创建默认机器人。 |
| `/poecreate / pc` | 创建机器人。 |
| `/poeremove / pr (+ 机器人名称)` | 删除指定名称的机器人。 |
| `/poeswitch / ps (+ 机器人名称)` | 切换到指定名称的机器人。 |
| `你的机器人的名字 + 空格 + 你要询问的内容` | 指定机器人对话。 |

************************

- 以下命令均是多用户共享的

| 命令 | 描述 |
| --- | --- |
| `/psn + 你要搜索的内容` | NeevaAI搜索引擎，返回链接及标题。 |
| `/psg + 你要询问的内容` | 共享的GPT对话。 |
| `/psc + 你要询问的内容` | 共享的CLAUDE对话。 |
| `/pss + 你要询问的内容` | 共享的SAGE对话。 |
| `/psd + 你要询问的内容` | 共享的Dragonfly对话。 |

************************

- 以下功能仅限特定白名单用户使用，同样是多用户共享的

| 命令 | 描述 |
| --- | --- |
| `/psg4 + 询问内容` | 使用GPT4对话。 |
| `/pscp + 询问内容` | 使用CLAUDE+对话。 |

************************

- 以下功能仅限poe管理员使用

| 命令 | 描述 |
| --- | --- |
| `/poechangeprompt / 切换自动预设 / pcp` | 切换自动创建的默认预设。 |
"""
    pic = await md_to_pic(msg)
    await poe_help.send(MessageSegment.image(pic))
    await poe_help.finish()



from nonebot import logger
from playwright.async_api import async_playwright, Page
from .config import poe_persistor


# 用元类实现单例，提供一个创建新页面的函数
class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class PlaywrightFramework(metaclass=Singleton):
    def __init__(self):
        self.context = None
        self.playwright = None
        self.browser = None

    async def init_framework(self):
        if not self.playwright:
            self.playwright = await async_playwright().start()
            cookies = [poe_persistor.cookie_dict]
            server = poe_persistor.server
            username = poe_persistor.username
            passwd = poe_persistor.passwd
            proxy_config = {}
            if server is not None:
                proxy_config["server"] = server
            if username is not None:
                proxy_config["username"] = username
            if passwd is not None:
                proxy_config["password"] = passwd
            if proxy_config:
                self.browser = await self.playwright.chromium.launch(proxy=proxy_config)
            else:
                self.browser = await self.playwright.chromium.launch()
            self.context = await self.browser.new_context(device_scale_factor=2)
            logger.info("初次创建context")
            if poe_persistor.cookie_dict:
                await self.context.add_cookies(cookies=cookies)

    async def close_framework(self):
        if self.context:
            await self.context.close()
            await self.browser.close()
            await self.playwright.stop()
            self.context = None
            self.browser = None
            self.playwright = None

    async def new_page(self) -> Page:
        if not self.context:
            await self.init_framework()
        return await self.context.new_page()


pwfw = PlaywrightFramework()



import json
from typing import Dict, List, Optional, Tuple
from bidict import bidict
import nonebot
from nonebot.adapters.onebot.v11 import MessageEvent
from pathlib import Path
from pydantic import BaseModel, Extra
from ..common.config import spark_persistor
import json


class BotInfo(BaseModel):
    # BotInfo本身储存就决定了它是单平台单来源的
    # 这里预留一些东西只是留作可能的修改使用
    nickname: Optional[str]
    truename: Optional[str]
    source: Optional[str]
    model: Optional[str]
    num_users: Optional[int]
    prompt_nickname: Optional[str]
    prompt: Optional[str]
    # 这里做owner主要是用来hash，以区分不同用户的可能重名的bot
    owner: Optional[str]

    last_suggests: Optional[List[str]] = []

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.nickname, self.truename, self.owner))

    @classmethod
    def from_dict(cls, data: dict) -> "BotInfo":
        return cls(**data)


class UserInfo(BaseModel):
    platform: str
    user_id: int

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.platform, self.user_id))

    @classmethod
    def from_dict(cls, data: dict) -> "UserInfo":
        return cls(**data)

    def to_string(self) -> str:
        return f"{self.platform}-{self.user_id}"

    @classmethod
    def from_string(cls, s: str) -> "UserInfo":
        platform, user_id = s.split("-")
        return cls(platform=platform, user_id=int(user_id))


class Sender(BaseModel, extra=Extra.ignore):
    user_id: int
    user_name: str

    def __hash__(self):
        return hash((self.user_id, self.user_name))

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "Sender":
        return cls(**data)


class UserData(BaseModel):
    sender: Sender
    wait_msg_id: Optional[int] = None
    last_reply_message_id: Dict[str, int] = {}
    is_waiting: bool = False

    def __hash__(self):
        return hash(
            (
                self.sender,
                self.wait_msg_id,
                self.last_reply_message_id,
                self.last_reply_message_id,
            )
        )

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "UserData":
        return cls(**data)


# 储存poe的临时数据
class PoeTemper(BaseModel):
    # 储存以用户为主体的信息，用户的info和data，只会用到单项查找
    user_data_dict: dict[UserInfo, UserData] = {}
    # 储存以bot为主体的信息，所有bot的msgid，和对应的bot信息,用bidict来方便操作唯一的键值对
    msg_bot_bidict: bidict[int, BotInfo] = bidict()
    # 储存poe网站的basebot作为公共bot,此数据长期不变，所以以临时方式储存
    base_botinfo_dict: dict[str, BotInfo] = {
        "psg4": BotInfo(
            truename="GPT-4", nickname="psg4", owner="_every_one_", num_users=0
        ),
        "psc+": BotInfo(
            truename="Claude+", nickname="psc+", owner="_every_one_", num_users=0
        ),
        "psg": BotInfo(
            truename="ChatGPT", nickname="psg", owner="_every_one_", num_users=0
        ),
        "psc": BotInfo(
            truename="Claude-instant", nickname="psc", owner="_every_one_", num_users=0
        ),
        "pss": BotInfo(
            truename="Sage", nickname="pss", owner="_every_one_", num_users=0
        ),
        "psd": BotInfo(
            truename="Dragonfly", nickname="psd", owner="_every_one_", num_users=0
        ),
        "psn": BotInfo(
            truename="NeevaAI", nickname="psn", owner="_every_one_", num_users=0
        ),
    }


def set_userdata(
    event: MessageEvent, user_data_dict: dict[UserInfo, UserData]
) -> UserData:
    """
    接受一个event和一个user_data_dict(全局的，所有用户的),在user_data_dict匹配(无则初次设置)用户的过往使用信息
    返回UserInfo和匹配后的(无则新建的)对应用户的current_user_dict结构为[UserInfo, UserData]
    """
    user_info = UserInfo(platform="qq", user_id=event.user_id)
    user_data = UserData(
        sender=Sender(
            user_id=event.user_id,
            user_name=event.sender.nickname or "<未知的的用户名>",
        )
    )
    return user_info, user_data_dict.setdefault(user_info, user_data)


def get_user_info_and_data(event: MessageEvent) -> Tuple[UserInfo, UserData]:
    """
    接受一个 event ，返回一个包含 UserInfo 和 UserData 的元组。
    仅仅将构造好的 UserInfo 和 UserData 构建成一个一个元组并返回。
    用来初次生成UserInfo和UserData
    """
    return (
        UserInfo(platform="qq", user_id=event.user_id),
        UserData(
            sender=Sender(
                user_id=event.user_id,
                user_name=event.sender.nickname or "<未知的的用户名>",
            )
        ),
    )


# 储存poe的持久数据,提供一键保存和加载功能
class PoePersistor(BaseModel):
    path_: str = str(Path()) + "/data/spark_gpt/poe_data.json"
    pic_able: str = None
    url_able: str = "True"
    suggest_able: str = "True"
    num_limit: int = 350
    server: str = None
    username: str = None
    passwd: str = None
    superusers: List[str] = []
    blacklist: List[str] = []
    whitelist: List[str] = []
    accesslist: List[str] = []
    mode: str = "black"
    poe_cookie: str = None
    cookie_dict: dict = {}
    # user_dict: Dict[UserInfo, Dict["all" or "now", Dict[nickname, BotInfo]]] = {}
    user_dict: Dict[UserInfo, Dict[str, Dict[str, BotInfo]]] = {}
    auto_prompt: str = "默认"

    def __init__(self, **data):
        super().__init__(**data)
        try:
            saved_data = self.load()
            self.__dict__.update(saved_data)
        except:
            pass
        get_config = nonebot.get_driver().config
        # 尝试读取个性配置，若无则从通用配置读取
        self.superusers = (
            get_config.poe_superusers
            if hasattr(get_config, "poe_superusers")
            else spark_persistor.superusers
        )
        self.blacklist = (
            get_config.poe_blacklist
            if hasattr(get_config, "poe_blacklist")
            else spark_persistor.blacklist
        )
        self.whitelist = (
            get_config.poe_whitelist
            if hasattr(get_config, "poe_whitelist")
            else spark_persistor.whitelist
        )
        self.accesslist = (
            get_config.poe_accesslist
            if hasattr(get_config, "poe_accesslist")
            else self.superusers
        )
        self.mode = (
            get_config.poe_mode
            if hasattr(get_config, "poe_mode")
            and get_config.poe_mode in ["white", "black"]
            else spark_persistor.mode
        )
        self.pic_able = getattr(get_config, "poe_picable", spark_persistor.pic_able)
        self.url_able = getattr(get_config, "poe_urlable", spark_persistor.url_able)
        self.suggest_able = getattr(
            get_config, "poe_suggestable", spark_persistor.suggest_able
        )
        self.num_limit = int(
            getattr(get_config, "poe_limit", spark_persistor.num_limit)
        )
        # 这是单独配置
        self.server = getattr(get_config, "poe_server", self.server)
        self.username = getattr(get_config, "poe_username", self.username)
        self.passwd = getattr(get_config, "poe_passwd", self.passwd)
        self.poe_cookie = getattr(get_config, "poe_cookie", self.poe_cookie)

        if self.poe_cookie:
            self.cookie_dict = {
                "domain": "poe.com",
                "name": "p-b",
                "path": "/",
                "value": f"{self.poe_cookie}",
            }
        self.save()

    def save(self):
        Path(self.path_).parent.mkdir(parents=True, exist_ok=True)
        new_user_dict = {}
        user_dict = self.user_dict
        for user, bot_dict in user_dict.items():
            new_user_dict[user.to_string()] = {}
            for class_name, bot_info_dict in bot_dict.items():
                new_user_dict[user.to_string()][class_name] = {}
                for nickname, bot_info in bot_info_dict.items():
                    new_user_dict[user.to_string()][class_name][
                        nickname
                    ] = bot_info.to_dict()
        data = self.__dict__.copy()
        data["user_dict"] = new_user_dict
        with open(self.path_, "w", encoding="utf-8") as f:
            json.dump(data, f, ensure_ascii=False)

    def load(self):
        with open(self.path_, "r", encoding="utf-8") as f:
            data = json.load(f)
        user_dict = data.get("user_dict", {})
        new_user_dict = {}
        for user_info_s, bot_infos in user_dict.items():
            userinfo = UserInfo.from_string(user_info_s)
            new_bots_infos = {}
            for class_name, bots in bot_infos.items():
                new_bots_infos[class_name] = {}
                for nickname, bot in bots.items():
                    new_bots_infos[class_name][nickname] = BotInfo.from_dict(bot)
            new_user_dict[userinfo] = new_bots_infos
        data["user_dict"] = new_user_dict
        return data


# 这里的配置即使在多平台，也应只初始化一次
poe_persistor = PoePersistor()



import asyncio
from nonebot.plugin import on_command, on_message
from nonebot.params import ArgStr, CommandArg
from nonebot.typing import T_State
from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import (
    Message,
    Event,
    Bot,
    MessageEvent,
    MessageSegment,
)


from .chatgpt_web_func import random_uuid4, is_useable
from .config import (
    BotInfo,
    GPT_webTemper,
    gptweb_persistor,
    get_user_info_and_data,
    set_userdata,
)
from .chatgpt_web_func import reply_out, delete_messages, sendmsg
from .web_api import gptweb_api
from ..common.config import spark_persistor
from ..poe.config import poe_persistor
from nonebot.rule import fullmatch
from nonebot import logger

# 初始化两个需要使用的实例
temp_data = GPT_webTemper()
user_data_dict = temp_data.user_data_dict
msg_bot_bidict = temp_data.msg_bot_bidict
prompts_dict = spark_persistor.prompts_dict
from ..common.render.render import md_to_pic

logger.info("开始加载gpt_web")


######################################################
creat_lock = asyncio.Lock()
gpt_web_create_ = on_command("gwcreate", aliases={"gwc"}, priority=4, block=False)


@gpt_web_create_.handle()
async def __(matcher: Matcher, state: T_State, event: Event):
    if not is_useable(event):
        await matcher.finish()
    create_msgs = []
    if len(prompts_dict) > 0:
        str_prompts = str()
        for key, _ in prompts_dict.items():
            str_prompts += f"{key}\n"
        # create_msgs.append(await matcher.send(reply_out(event, f"当前预设有：\n{str_prompts}")))
        msg = f'当前预设有：\n{str_prompts}\n请输入\n1.机器人名称,\n2.自定义预设（预设内容中间不要有空格） 或 "." + 可用本地预设名\n两个参数中间用空格隔开\n最终格式示例:\n示例1:猫娘 .猫娘\n示例2:chat 一个ai语言模型\n输入取消 或 算了可以终止创建'
    create_msgs.append(await matcher.send(reply_out(event, msg)))
    state["create_msgs"] = create_msgs


@gpt_web_create_.got("model")
async def __gpt_web_create___(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    infos: str = ArgStr("model"),
):
    global gptweb_persistor
    create_msgs = state["create_msgs"]
    user_id = str(event.user_id)
    infos = infos.split(" ", 2)
    if infos[0] in ["取消", "算了"]:
        create_msgs.append(await matcher.send(reply_out(event, "取消创建")))
        await delete_messages(bot, str(event.user_id), create_msgs)
        await matcher.finish()
    if len(infos) != 2:
        create_msgs.append(await matcher.reject(reply_out(event, "你输入的信息有误，请重新输入")))

    nickname = str(infos[0])
    truename = None
    parentname = str(random_uuid4())
    prompt = str(infos[1])

    prompt_name = str("unknown")
    if prompt.startswith("."):
        prompt_name = prompt[1:]
        if prompt_name in prompts_dict:
            prompt = prompts_dict[prompt_name]
        else:
            create_msgs.append(
                await matcher.send(reply_out(event, "输入的本地预设名不正确，请重新输入"))
            )
            await matcher.reject()

    current_userinfo, current_userdata = set_userdata(event, user_data_dict)
    if current_userinfo not in list(gptweb_persistor.user_dict.keys()):
        gptweb_persistor.user_dict.setdefault(current_userinfo, {"all": {}, "now": {}})

    # 查看对应用户下是不是有重名的bot
    if (
        current_userinfo in gptweb_persistor.user_dict
        and nickname in gptweb_persistor.user_dict[current_userinfo]["all"]
    ) or (
        current_userinfo in poe_persistor.user_dict
        and nickname in poe_persistor.user_dict[current_userinfo]["all"]
    ):
        create_msgs.append(
            await matcher.send(reply_out(event, "已经有同名的bot了，换一个名字重新输入吧"))
        )
        await matcher.reject()
    if creat_lock.locked():
        waitmsg = await matcher.send(reply_out(event, "有人正在创建中，稍后自动为你创建"))
    async with creat_lock:
        try:
            result = await gptweb_api.gpt_web_chat(truename, parentname, prompt)
        except:
            await matcher.send(reply_out(event, "出错了，多次出错请尝试换一个预设，还不行请联系机器人主人"))
            await delete_messages(bot, user_id, create_msgs)
        if isinstance(result, str):
            text_error = result
            await matcher.finish(reply_out(event, text_error))
        elif isinstance(result, tuple):
            answer, parentname, truename = result
            # 将更新后的字典写回到JSON文件中
            botinfo = BotInfo(
                nickname=nickname,
                truename=truename,
                parentname=parentname,
                source="gpt_web",
                prompt_nickname=prompt_name,
                prompt=prompt,
                owner="qq-" + str(event.user_id),
            )
            gptweb_persistor.user_dict.setdefault(current_userinfo, {}).setdefault(
                "all", {}
            )[nickname] = botinfo

            gptweb_persistor.user_dict[current_userinfo]["now"] = {nickname: botinfo}

            gptweb_persistor.save()
            try:
                await bot.delete_msg(message_id=waitmsg["message_id"])
            except:
                pass
            reply_msgid = await matcher.send(
                reply_out(event, f"创建成功并切换到新建bot\n\n创建自动回复:{answer}")
            )
            msg_bot_bidict[reply_msgid["message_id"]] = botinfo
            current_userdata.last_reply_message_id[nickname] = reply_msgid["message_id"]
            await delete_messages(bot, user_id, create_msgs)
            await matcher.finish()
        else:
            await matcher.send(reply_out(event, "出错了，多次出错请联系机器人管理员"))

            await delete_messages(bot, user_id, create_msgs)
            await matcher.finish()


#############################################################
gptweb_switch = on_command("gwswitch", aliases={"gws"}, priority=4, block=False)


@gptweb_switch.handle()
async def __gptweb_switch__(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    args: Message = CommandArg(),
):
    if not is_useable(event):
        await matcher.finish()
    userinfo, _ = get_user_info_and_data(event)
    switch_msgs = []
    if userinfo not in list(gptweb_persistor.user_dict.keys()):
        await matcher.finish(reply_out(event, "你还没创建任何bot"))
    if args:
        nickname = str(args[0])
        if nickname not in gptweb_persistor.user_dict[userinfo]["all"].keys():
            switch_msgs.append(await matcher.finish(reply_out(event, "没有这个机器人呢")))

        gptweb_persistor.user_dict[userinfo]["now"] = {
            nickname: gptweb_persistor.user_dict[userinfo]["all"][nickname]
        }
        gptweb_persistor.save()
        await matcher.send(reply_out(event, f"已切换为{nickname}"))
        await delete_messages(bot, userinfo, switch_msgs)
        await gptweb_switch.finish()

    gw_bots = list(gptweb_persistor.user_dict[userinfo]["all"].keys())
    gw_bot_str = "\ngpt_web机器人有:\n" + "\n".join(str(bot) for bot in gw_bots)
    nickname = str(list(gptweb_persistor.user_dict[userinfo]["now"].keys())[0])
    if len(gw_bots) == 1:
        await matcher.finish(reply_out(event, f"当前只有一个bot:{nickname},无法切换"))
    msg = (
        "你已经创建的的bot有：\n"
        + gw_bot_str
        + f"\n当前使用的bot是{nickname}\n\n请输入要切换的机器人名称\n输入取消 或 算了可以终止创建"
    )
    switch_msgs.append(await matcher.send(reply_out(event, msg)))
    state["switch_msgs"] = switch_msgs
    state["userinfo"] = userinfo


@gptweb_switch.got("nickname")
async def __gptweb_switch____(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    infos: str = ArgStr("nickname"),
):
    switch_msgs = state["switch_msgs"]
    userinfo = state["userinfo"]

    if infos in ["取消", "算了"]:
        switch_msgs.append(await matcher.send(reply_out(event, "中断切换")))
        await delete_messages(bot, userinfo, switch_msgs)
        await matcher.finish()

    nickname = infos.split(" ")[0]
    if nickname not in list(gptweb_persistor.user_dict[userinfo]["all"].keys()):
        switch_msgs.append(await matcher.send(reply_out(event, "没有这个机器人呢")))
        await gptweb_switch.reject()

    gptweb_persistor.user_dict[userinfo]["now"] = {
        nickname: gptweb_persistor.user_dict[userinfo]["all"][nickname]
    }
    gptweb_persistor.save()
    await matcher.send(reply_out(event, f"已切换为{nickname}"))
    await delete_messages(bot, userinfo, switch_msgs)
    await gptweb_switch.finish()


#############################################################
async def _is_chat_(event: MessageEvent, bot: Bot):
    # 处理用命令来进行的对话,包括pt，指定名，公共的，也可以处理回复形式的对话
    try:
        current_userinfo, current_userdata = set_userdata(
            event=event, user_data_dict=user_data_dict
        )
    except:
        return False
    if bool(event.reply):
        if str(event.reply.sender.user_id) == bot.self_id:
            raw_message = str(event.message)
            if not raw_message:
                return False
            try:
                botinfo = msg_bot_bidict[event.reply.message_id]
                if botinfo.nickname in current_userdata.last_reply_message_id:
                    if not is_useable(event):
                        return False
                    return (
                        event.reply.message_id,
                        raw_message,
                        botinfo,
                        current_userinfo,
                        current_userdata,
                    )
                else:
                    return False
            except:
                return False
        else:
            return False
    elif str(event.message).startswith(("/gwt", "/gwtalk")):
        if not is_useable(event):
            return False
        raw_message = (
            str(event.message)
            .replace("/gwtalk ", "")
            .replace("/gwtalk", "")
            .replace("/gwt ", "")
            .replace("/gwt", "")
        )
        if not raw_message:
            return False
        try:
            botinfo = list(
                gptweb_persistor.user_dict[current_userinfo]["now"].values()
            )[0]
            nickname = botinfo.nickname
        except:
            return "none"
        try:
            last_msgid = current_userdata.last_reply_message_id[nickname]
        except:
            last_msgid = 0
        return last_msgid, raw_message, botinfo, current_userinfo, current_userdata
    else:
        try:
            bots_nicknames = list(
                gptweb_persistor.user_dict[current_userinfo]["all"].keys()
            )
            nickname = next(
                (
                    name
                    for name in bots_nicknames
                    if str(event.message).startswith("/" + name)
                ),
                None,
            )
            if nickname:
                if not is_useable(event):
                    return False
                botinfo = gptweb_persistor.user_dict[current_userinfo]["all"][
                    nickname
                ]
                raw_message = (
                    str(event.message)
                    .replace("/" + nickname + " ", "")
                    .replace("/" + nickname, "")
                )
                if not raw_message:
                    return False
                try:
                    last_msgid = current_userdata.last_reply_message_id[nickname]
                except:
                    last_msgid = 0
                return (
                    last_msgid,
                    raw_message,
                    botinfo,
                    current_userinfo,
                    current_userdata,
                )
            else:
                return False
        except:
            return False



#############################################################
chat_lock = asyncio.Semaphore(3)

gw_chat_ = on_message(priority=1, block=False)


@gw_chat_.handle()
async def __chat_bot__(matcher: Matcher, event: MessageEvent, bot: Bot):
    temp = await _is_chat_(event, bot)
    if temp == False:
        await matcher.finish()
    if temp == "none":
        nickname = "gwdefault"
        truename = None
        parentname = str(random_uuid4())
        prompt_nickname = gptweb_persistor.auto_prompt
        prompt = prompts_dict[prompt_nickname]
        raw_message = (
            str(event.message)
            .replace("/gwtalk ", "")
            .replace("/gwtalk", "")
            .replace("/gwt ", "")
            .replace("/gwt", "")
        )
        lastmsg_id = 0
        current_userinfo, current_userdata = set_userdata(event, user_data_dict)
        if current_userinfo not in list(gptweb_persistor.user_dict.keys()):
            gptweb_persistor.user_dict.setdefault(
                current_userinfo, {"all": {}, "now": {}}
            )

        if creat_lock.locked():
            waitmsg = await matcher.send(reply_out(event, "有人正在创建中，稍后自动为你创建"))
        async with creat_lock:
            try:
                result = await gptweb_api.gpt_web_chat(truename, parentname, prompt)
            except:
                await matcher.finish(reply_out(event, "出错了，多次出错请联系机器人管理员"))
            if isinstance(result, str):
                text_error = result
            elif isinstance(result, tuple):
                answer, parentname, truename = result
                # 将更新后的字典写回到JSON文件中
                botinfo = BotInfo(
                    nickname=nickname,
                    truename=truename,
                    parentname=parentname,
                    source="gpt_web",
                    prompt_nickname=prompt_nickname,
                    prompt=prompt,
                    owner="qq-" + str(event.user_id),
                )
                gptweb_persistor.user_dict.setdefault(current_userinfo, {}).setdefault(
                    "all", {}
                )[nickname] = botinfo
                gptweb_persistor.user_dict.setdefault(current_userinfo, {}).setdefault(
                    "now", {}
                )[nickname] = botinfo
                gptweb_persistor.save()
                try:
                    await bot.delete_msg(message_id=waitmsg["message_id"])
                except:
                    pass
                await matcher.send(
                    reply_out(event, f"自动创建成功并切换到新建bot:gwdefault\n自动创建回复:\n{answer}")
                )
            else:
                await matcher.finish(reply_out(event, "出错了，多次出错请联系机器人管理员"))
    else:
        lastmsg_id, raw_message, botinfo, current_userinfo, current_userdata = temp
    nickname = botinfo.nickname
    truename = botinfo.truename
    parentname = botinfo.parentname
    if current_userdata.is_waiting:
        await matcher.finish(reply_out(event, "你已经有一个请求进行中了，请等结束后再发送"))
    if chat_lock.locked():
        await matcher.send(reply_out(event, "请稍等,你前面已有3个用户,你的回答稍后就来"))

    async with chat_lock:
        if raw_message in [
            "清除对话",
            "清空对话",
            "清除历史",
            "清空历史",
            "清除记录",
            "清空历史对话",
            "刷新对话",
            "pd",
            "gwdump",
        ]:
            truename = None
            parentname = str(random_uuid4())
            prompt_nickname = gptweb_persistor.auto_prompt
            prompt = prompts_dict[prompt_nickname]

            if current_userinfo not in list(gptweb_persistor.user_dict.keys()):
                gptweb_persistor.user_dict.setdefault(
                    current_userinfo, {"all": {}, "now": {}}
                )

            if creat_lock.locked():
                waitmsg = await matcher.send(reply_out(event, "请稍等，马上就好"))
            async with creat_lock:
                current_userdata.is_waiting = True
                result = await gptweb_api.gpt_web_chat(truename, parentname, prompt)
                current_userdata.is_waiting = False
                if isinstance(result, str):
                    text_error = result
                elif isinstance(result, tuple):
                    answer, parentname, truename = result
                    # 将更新后的字典写回到JSON文件中
                    botinfo = BotInfo(
                        nickname=nickname,
                        truename=truename,
                        parentname=parentname,
                        source="gpt_web",
                        prompt_nickname=prompt_nickname,
                        prompt=prompt,
                        owner="qq-" + str(event.user_id),
                    )
                    gptweb_persistor.user_dict[current_userinfo]["all"][
                        nickname
                    ] = botinfo
                    if botinfo.nickname == list(gptweb_persistor.user_dict[current_userinfo]["now"].values())[0].nickname:
                        gptweb_persistor.user_dict[current_userinfo]["now"] = {
                            nickname: botinfo
                        }
                    gptweb_persistor.save()
                    msg_bot_bidict[lastmsg_id] = botinfo
                    try:
                        await bot.delete_msg(message_id=waitmsg["message_id"])
                    except:
                        pass
                    reply_msgid = await matcher.send(
                        reply_out(event, f"刷新对话成功\n刷新回复:\n{answer}")
                    )
                    current_userdata.last_reply_message_id[nickname] = reply_msgid[
                        "message_id"
                    ]
                    msg_bot_bidict.inv[botinfo] = reply_msgid[
                        "message_id"
                    ]
                    await matcher.finish()
                else:
                    await matcher.finish(reply_out(event, "出错了，多次出错请联系机器人管理员"))
        else:
            current_userdata.is_waiting = True
            try:
                result = await gptweb_api.gpt_web_chat(
                    truename, parentname, raw_message
                )
            except:
                current_userdata.is_waiting = False
                await matcher.finish(reply_out(event, "出错了，多次出错请联系机器人主人"))
            current_userdata.is_waiting = False
            if isinstance(result, str):
                text_error = result
                logger.warning(text_error)
                await matcher.send(reply_out(event, text_error))
            elif isinstance(result, tuple):
                answer, parentname, truename = result
                botinfo.truename = truename
                botinfo.parentname = parentname

                msg_bot_bidict[lastmsg_id] = botinfo

                gptweb_persistor.user_dict[current_userinfo]["all"][nickname] = botinfo
                if botinfo in gptweb_persistor.user_dict[current_userinfo]["now"]:
                    gptweb_persistor.user_dict[current_userinfo]["now"] = { 
                        nickname: botinfo
                    }
                gptweb_persistor.save()

                reply_msgid = await sendmsg(answer, matcher, event)
                current_userdata.last_reply_message_id[nickname] = reply_msgid[
                    "message_id"
                ]

                msg_bot_bidict.inv[botinfo] = reply_msgid["message_id"]
                await matcher.finish()


# ######################################################
gptweb_remove = on_command("gwremove", aliases={"gwr"}, priority=4, block=False)


@gptweb_remove.handle()
async def __gptweb_remove__(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    args: Message = CommandArg(),
):
    if not is_useable(event):
        await matcher.finish()
    userinfo, _ = get_user_info_and_data(event)

    remove_msgs = []
    if userinfo not in list(gptweb_persistor.user_dict.keys()):
        await matcher.finish(reply_out(event, "你还没创建任何bot"))

    # 如果有直接的arg的nickname，直接删除它，否则跳过
    if str(args):
        nickname = str(args[0])
        if nickname not in gptweb_persistor.user_dict[userinfo]["all"]:
            await matcher.finish(reply_out(event, "没有这个机器人呢"))
        if nickname == str(list(gptweb_persistor.user_dict[userinfo]["now"].keys())[0]):
            await matcher.finish(reply_out(event, "不能删除正在使用的bot哦"))
        del gptweb_persistor.user_dict[userinfo]["all"][nickname]
        msg = f"已删除{nickname}"
        await matcher.finish(reply_out(event, msg))

    bots = list(gptweb_persistor.user_dict[userinfo]["all"].keys())
    if len(bots) == 1:
        await matcher.finish(reply_out(event, f"当前只有一个机器人:{bots[0]},不能删除"))
    bot_nickname_str = "\n".join(str(bot) for bot in bots)

    nickname = str(list(gptweb_persistor.user_dict[userinfo]["now"].keys())[0])
    msg = (
        "你已经创建的的bot有：\n"
        + bot_nickname_str
        + f"\n当前使用的bot是{nickname}\n\n请输入要删除的机器人名称\n输入取消 或 算了可以终止创建"
    )
    remove_msgs.append(await matcher.send(reply_out(event, msg)))
    state["remove_msgs"] = remove_msgs
    state["userinfo"] = userinfo


@gptweb_remove.got("nickname")
async def __gptweb_remove____(
    bot: Bot,
    matcher: Matcher,
    event: Event,
    state: T_State,
    infos: str = ArgStr("nickname"),
):
    if not is_useable(event):
        await matcher.finish()
    userinfo = state["userinfo"]
    remove_msgs = state["remove_msgs"]
    bots = list(gptweb_persistor.user_dict[userinfo]["all"].keys())
    if infos in ["取消", "算了"]:
        remove_msgs.append(await matcher.send(reply_out(event, "终止删除")))

        await delete_messages(bot, str(event.user_id), remove_msgs)
        await matcher.finish()
    infos = infos.split(" ")
    nickname_delete = infos[0]
    nickname_now = str(list(gptweb_persistor.user_dict[userinfo]["now"].keys())[0])
    if not (nickname_delete in bots):
        remove_msgs.append(await matcher.send(reply_out(event, "输入信息有误，请检查后重新输入")))
        await gptweb_remove.reject()
    if nickname_delete == nickname_now:
        remove_msgs.append(await matcher.send(reply_out(event, "不能删除正在使用的bot哦")))

        await delete_messages(bot, userinfo, remove_msgs)
        await gptweb_remove.finish()
    del gptweb_persistor.user_dict[userinfo]["all"][nickname_delete]
    gptweb_persistor.save()

    await matcher.send(reply_out(event, f"已删除{nickname_delete}"))
    await delete_messages(bot, userinfo, remove_msgs)
    await matcher.finish()


######################################################
gw_auto_change_prompt = on_command(
    "gwchangeprompt", aliases={"gwcp"}, priority=4, block=False
)


@gw_auto_change_prompt.handle()
async def __poe_auto_change_prompt__(matcher:Matcher,event: Event):
    if not is_useable(event):
        await matcher.finish()
    global gptweb_persistor
    user_id = str(event.user_id)
    if user_id not in gptweb_persistor.superusers:
        await gw_auto_change_prompt.finish("你不是管理员哦")
    now_prompt = gptweb_persistor.auto_prompt
    str_prompts = str()
    i = 1
    for key, value in prompts_dict.items():
        str_prompts += f"*******************\n{i}:预设名称：{key}\n预设内容：{value}\n"
        i += 1
    await gw_auto_change_prompt.send(
        f"现在的自动创建预设是:{now_prompt}\n当前可用预设有：\n{str_prompts}"
    )


@gw_auto_change_prompt.got("name", prompt="请输入要切换到的预设名称\n输入取消 或 算了可以终止创建")
async def __poe_auto_change_prompt____(
    event: Event, state: T_State, infos: str = ArgStr("name")
):
    if infos in ["取消", "算了"]:
        await gw_auto_change_prompt.finish("终止切换")
    infos = infos.split(" ")
    if len(infos) != 1 or infos[0] not in prompts_dict:
        await gw_auto_change_prompt.reject("你输入的信息有误，请检查后重新输入")
    # 将更新后的字典写回到JSON文件中
    gptweb_persistor.auto_prompt = infos[0]
    gptweb_persistor.save()
    await gw_auto_change_prompt.finish("成功切换默认自动创建prompt")


######################################################
gw_help = on_command("gwhelp", aliases={"gw帮助", "gwh"}, priority=4, block=False)


@gw_help.handle()
async def __gw_help__(bot: Bot, matcher: Matcher, event: Event):
    user_id = str(event.user_id)
    if not is_useable(event):
        await matcher.finish()
    msg = """
# spark-gpt GPT_web使用说明

- !!! 以下命令前面全部要加 '/' !!!  

- 问答功能均支持以下特性：可以通过回复机器人的最后一个回答来继续对话，而无需命令；可以回复 "(清除/清空)(对话/历史)"或 刷新对话，清除对话历史 或 "pd","poedump","pdump"来清空对话；可以通过建议回复的数字索引来使用建议回复。

## 对话命令

| 命令 | 描述 |
| --- | --- |
| `/gwtalk / gwt + 你要询问的内容` | 对话功能，如果没创建机器人，对话将自动创建默认机器人。 |

## 机器人管理命令

| 命令 | 描述 |
| --- | --- |
| `/gwcreate / gwc` | 创建机器人。 |
| `/gwremove / gwr (+ 机器人名称)` | 删除指定名称的机器人。 |
| `/gwswitch / gws (+ 机器人名称)` | 切换到指定名称的机器人。 |
| `/机器人名字 + 空格 + 你要询问的内容` | 指定机器人对话。 |

## 管理员命令

- 仅限poe管理员使用

| 命令 | 描述 |
| --- | --- |
| `/gwcp / gwchangeprompt` | 切换自动创建的默认预设。 |"""
    pic = await md_to_pic(msg)
    await gw_help.send(MessageSegment.image(pic))
    await gw_help.finish()



import re
from .config import newbing_persistor
from nonebot import logger
from typing import Union
from nonebot import logger, require

require("nonebot_plugin_guild_patch")
from nonebot_plugin_guild_patch import GuildMessageEvent
from nonebot.adapters.onebot.v11 import Message, MessageEvent, MessageSegment
from ..common.render.render import md_to_pic
from ..common.common_func import get_url


def is_useable(event, mode=newbing_persistor.mode):
    if mode == "black":
        try:
            if str(event.user_id) in newbing_persistor.blacklist:
                logger.warning(f"黑名单用户:{str(event.user_id)},跳过")
                return False
        except:
            pass
        try:
            if str(event.group_id) in newbing_persistor.blacklist:
                logger.warning(f"黑名单群聊:{str(event.group_id)},跳过")
                return False
        except:
            pass
        return True
    elif mode == "white":
        try:
            if str(event.user_id) in newbing_persistor.whitelist:
                return True
        except:
            pass
        try:
            if str(event.group_id) in newbing_persistor.whitelist:
                return True
        except:
            pass
        logger.warning("用户或群聊不在白名单内，跳过")
        return False


def reply_out(
    event: MessageEvent, content: Union[MessageSegment, Message, str, bytes]
) -> Message:
    """返回一个回复消息"""
    if isinstance(event, GuildMessageEvent):
        return Message(content)
    if type(content) == bytes:
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    if content[0:9] == "base64://":
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    return MessageSegment.reply(event.message_id) + content


async def sendmsg(msg, matcher, event):
    if newbing_persistor.pic_able is not None:
        if newbing_persistor.pic_able == "True":
            pic = await md_to_pic(msg)
            if newbing_persistor.url_able == "True":
                url = await get_url(msg)
                msgid_container = await matcher.send(
                    reply_out(event, pic) + MessageSegment.text(url)
                )
            else:
                msgid_container = await matcher.send(reply_out(event, pic))
        else:
            msgid_container = await matcher.send(reply_out(event, pic))
    else:
        if len(msg) > newbing_persistor.num_limit:
            pic = await md_to_pic(msg)
            if newbing_persistor.url_able == "True":
                url = await get_url(msg)
                msgid_container = await matcher.send(
                    reply_out(event, pic) + MessageSegment.text(url)
                )
            else:
                msgid_container = await matcher.send(reply_out(event, pic))
        else:
            msgid_container = await matcher.send(reply_out(event, msg))
    return msgid_container



import datetime
from typing import Union
import re, uuid, asyncio, string, random
from .config import poe_persistor
from nonebot import logger, require

require("nonebot_plugin_guild_patch")
from nonebot_plugin_guild_patch import GuildMessageEvent
from nonebot.adapters.onebot.v11 import Message, MessageEvent, MessageSegment
from ..common.render.render import md_to_pic
from ..common.common_func import get_url


async def if_quit(msg, matcher):
    if msg in ["取消", "算了"]:
        await matcher.send("终止添加")


def reply_out(
    event: MessageEvent, content: Union[MessageSegment, Message, str, bytes]
) -> Message:
    """返回一个回复消息"""
    if isinstance(event, GuildMessageEvent):
        return Message(content)
    if type(content) == bytes:
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    if content[0:9] == "base64://":
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    return MessageSegment.reply(event.message_id) + content


# 生成一个由qq号和nickname共同决定的uuid作为真名，防止重名
def generate_truename(user_id, nickname) -> str:
    current_time = datetime.datetime.now()
    current_time_str = current_time.strftime("%Y-%m-%d %H:%M:%S")
    s = user_id + nickname + current_time_str
    # 将字符串转换为 UUID 对象
    uuid_object = uuid.uuid3(uuid.NAMESPACE_DNS, s)
    # 获取 UUID 对象的 bytes 值
    uuid_bytes = uuid_object.bytes
    # 将 bytes 值转换为字符串
    uuid_str = uuid_bytes.hex()[:14]
    return uuid_str


def generate_random_string(length=8) -> str:
    """生成指定长度的随机字符串"""
    letters = string.ascii_letters + string.digits
    return "".join(random.choice(letters) for _ in range(length))


def is_email(email) -> bool:
    pattern = r"^[\w\.-]+@[\w\.-]+\.[a-zA-Z]{2,}$"
    return bool(re.match(pattern, email))


async def delete_messages(bot, user_id: str, dict_list: dict):
    await asyncio.sleep(1)
    if user_id in dict_list:
        if isinstance(dict_list[user_id], list):
            for eachmsg in dict_list[user_id]:
                await bot.delete_msg(message_id=eachmsg["message_id"])
            del dict_list[user_id]
        else:
            await bot.delete_msg(message_id=dict_list[user_id]["message_id"])
    else:
        for eachmsg in dict_list:
            await bot.delete_msg(message_id=eachmsg["message_id"])
        del dict_list


async def mdlink_2_str(md_text):
    result = []
    pattern = r"\[([^\]]+)\]\(([^)]+)\)(\s*-\s*([^[]+))?"  # 匹配超链接及其描述
    matches = re.findall(pattern, md_text)
    for i, match in enumerate(matches):
        # 提取标题
        title = match[0].strip() if not match[3] else match[3].strip()
        # 组合字符串
        result.append(f"{i+1}. {title} - {match[1].strip()}")

    output_str = "\n\n".join(result)
    return output_str


def is_useable(event, mode=poe_persistor.mode):
    if poe_persistor.poe_cookie:
        if mode == "black":
            try:
                if str(event.user_id) in poe_persistor.blacklist:
                    logger.warning(f"黑名单用户:{str(event.user_id)},跳过")
                    return False
            except:
                pass
            try:
                if str(event.group_id) in poe_persistor.blacklist:
                    logger.warning(f"黑名单群聊:{str(event.group_id)},跳过")
                    return False
            except:
                pass
            return True
        elif mode == "white":
            try:
                if str(event.user_id) in poe_persistor.whitelist:
                    return True
            except:
                pass
            try:
                if str(event.group_id) in poe_persistor.whitelist:
                    return True
            except:
                pass
            logger.warning("用户或群聊不在白名单内，跳过")
            return False
    else:
        logger.warning("没有配置poe的cookie，无法使用，跳过")
        return False


def is_vip(event):
    try:
        if str(event.user_id) in poe_persistor.accesslist:
            return True
    except:
        pass
    try:
        if str(event.group_id) in poe_persistor.accesslist:
            return True
    except:
        pass
    logger.warning("非vip用户或群聊，跳过")
    return False


async def close_page(page):
    try:
        await page.close()
    except:
        pass


async def send_msg(result, matcher, event):
    msgid_container = {}
    suggest_container = []

    async def sendmsg(msg, matcher, event):
        if poe_persistor.pic_able is not None:
            if poe_persistor.pic_able == "True":
                pic = await md_to_pic(msg)
                if poe_persistor.url_able == "True":
                    url = await get_url(msg)
                    msgid_container = await matcher.send(
                        reply_out(event, pic) + MessageSegment.text(url)
                    )
                else:
                    msgid_container = await matcher.send(reply_out(event, pic))
            else:
                msgid_container = await matcher.send(reply_out(event, pic))
        else:
            if len(msg) > poe_persistor.num_limit:
                pic = await md_to_pic(msg)
                if poe_persistor.url_able == "True":
                    url = await get_url(msg)
                    msgid_container = await matcher.send(
                        reply_out(event, pic) + MessageSegment.text(url)
                    )
                else:
                    msgid_container = await matcher.send(reply_out(event, pic))
            else:
                msgid_container = await matcher.send(reply_out(event, msg))
        return msgid_container

    if isinstance(result, tuple):
        last_answer, suggest_container = result
        is_successful = True
    elif isinstance(result, str):
        if "banned" == result:
            await matcher.send(reply_out(event, "你的机器人被banned了，请/pc新建一个机器人，并且不要在使用此预设"))
            matcher.finish()
            return msgid_container, []
        elif "limited" == result:
            await matcher.send(reply_out(event, "今日免费限额已用尽，请订阅或等明天再使用"))
            matcher.finish()
            return msgid_container, []
    elif isinstance(result, bool):
        is_successful = result
    else:
        raise ValueError("未知错误")

    if is_successful:
        if poe_persistor.suggest_able == "True" and suggest_container:
            suggest_str = "  \n".join(
                [f"{i+1}: {s}" for i, s in enumerate(suggest_container)]
            )
            msg = f"{last_answer}  \n\n建议回复：  \n{suggest_str}"
        else:
            msg = f"{last_answer}\n"

        msgid_container = await sendmsg(msg, matcher, event)
        return msgid_container, suggest_container
    else:
        await matcher.send(reply_out(event, "出错了，多次出错请联系机器人管理员"))
        return msgid_container, suggest_container



import json
from urllib.parse import urljoin
import httpx
from nonebot import logger
from .config import gptweb_persistor
from .chatgpt_web_func import random_uuid4

SESSION_TOKEN_KEY = "__Secure-next-auth.session-token"
CF_CLEARANCE_KEY = "cf_clearance"


class ChatGPT_web:
    def __init__(self) -> None:
        self.session_token = gptweb_persistor.session_token
        self.model = gptweb_persistor.model
        self.api_url = gptweb_persistor.api_url
        self.timeout = 30
        self.authorization = ""
        self.proxies = gptweb_persistor.proxy
        if self.session_token:
            self.auto_auth = False
        else:
            logger.warning("Chatgpt_web 未配置 session_token，无法使用")
        if self.api_url.startswith("https://chat.openai.com"):
            raise ValueError("无法使用官方API，请使用第三方API")

    async def gpt_web_chat(self, truename, parentname, text: str) -> str:
        if not self.authorization:
            await self.refresh_session()
            if not self.authorization:
                return "Token获取失败，请检查配置或API是否可用"
        async with httpx.AsyncClient(proxies=self.proxies) as client:
            async with client.stream(
                "POST",
                urljoin(self.api_url, "backend-api/conversation"),
                headers={
                    "Accept": "text/event-stream",
                    "Authorization": f"Bearer {self.authorization}",
                    "Content-Type": "application/json",
                    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15",
                    "X-Openai-Assistant-App-Id": "",
                    "Connection": "close",
                    "Accept-Language": "en-US,en;q=0.9",
                    "Referer": "https://chat.openai.com/chat",
                },
                json={
                    "action": "next",
                    "messages": [
                        {
                            "id": random_uuid4(),
                            "author": {"role": "user"},
                            "role": "user",
                            "content": {"content_type": "text", "parts": [text]},
                        }
                    ],
                    "conversation_id": truename,
                    "parent_message_id": parentname,
                    "model": "",
                    "timezone_offset_min": -480,
                },
                timeout=self.timeout,
            ) as response:
                if response.status_code == 429:
                    msg = ""
                    _buffer = bytearray()
                    async for chunk in response.aiter_bytes():
                        _buffer.extend(chunk)
                    resp: dict = json.loads(_buffer.decode())
                    if detail := resp.get("detail"):
                        if isinstance(detail, str):
                            msg += "\n" + detail
                    return "请求过多，请放慢速度" + msg
                if response.status_code == 401:
                    return "token失效，请重新设置token"
                if response.status_code == 403:
                    return "API错误，请联系开发者修复"
                if response.status_code == 404:
                    return "会话不存在"
                if response.is_error:
                    _buffer = bytearray()
                    async for chunk in response.aiter_bytes():
                        _buffer.extend(chunk)
                    resp_text = _buffer.decode()
                    logger.opt(colors=True).error(
                        f"非预期的响应内容: <r>HTTP{response.status_code}</r> {resp_text}"
                    )
                    return f"ChatGPT 服务器返回了非预期的内容: HTTP{response.status_code}\n{resp_text[:256]}"
                data_list = []
                async for line in response.aiter_lines():
                    if line.startswith("data:"):
                        data = line[6:]
                        if data.startswith("{"):
                            try:
                                data_list.append(json.loads(data))
                            except Exception as e:
                                logger.warning(f"ChatGPT数据解析未知错误：{e}: {data}")
                if not data_list:
                    return "ChatGPT 服务器未返回任何内容"
                idx = -1
                while data_list[idx]["error"]:
                    idx -= 1
                answer_json = data_list[idx]
                answer_text = answer_json["message"]["content"]["parts"][0]
                parent_id = answer_json["message"]["id"]
                conversation_id = answer_json["conversation_id"]
                return answer_text, parent_id, conversation_id

    async def refresh_session(self) -> None:
        cookies = {
            SESSION_TOKEN_KEY: self.session_token,
        }
        async with httpx.AsyncClient(
            cookies=cookies,
            proxies=self.proxies,
            timeout=self.timeout,
        ) as client:
            response = await client.get(
                urljoin(self.api_url, "api/auth/session"),
                headers={
                    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15",
                },
            )
            try:
                self.session_token = (
                    response.cookies.get(SESSION_TOKEN_KEY) or self.session_token
                )
                self.authorization = response.json()["accessToken"]
            except Exception as e:
                logger.opt(colors=True, exception=e).error(
                    f"刷新会话失败: <r>HTTP{response.status_code}</r> {response.text}"
                )

gptweb_api = ChatGPT_web()



from nonebot.plugin import on_command
from nonebot.params import ArgStr
from nonebot.typing import T_State
from nonebot.matcher import Matcher
from nonebot.adapters.onebot.v11 import Event,Message
from nonebot.params import ArgStr, CommandArg
from .render.render import md_to_pic
from .common_func import reply_out
from .config import spark_persistor, get_user_info_and_data
from .common_func import is_auto_prompt
from ..chatgpt_web.config import gptweb_persistor
from ..poe.config import poe_persistor

spark_addprompt = on_command("添加预设", aliases={"ap"}, priority=4, block=False)


@spark_addprompt.handle()
async def __spark_addprompt__(matcher: Matcher, event: Event):
    global spark_persistor
    user_id = str(event.user_id)
    if user_id not in spark_persistor.superusers:
        await spark_addprompt.finish("你不是管理员哦")


@spark_addprompt.got("name", prompt="请输入预设名称\n输入取消 或 算了可以终止创建")
async def __spark_addprompt____(
    event: Event, state: T_State, infos: str = ArgStr("name")
):
    global spark_persistor
    if infos in ["取消", "算了"]:
        await spark_addprompt.finish("终止添加")
    infos = infos.split(" ")
    if len(infos) != 1:
        await spark_addprompt.reject("你输入的信息有误，请检查后重新输入")
    state["key"] = infos[0]


@spark_addprompt.got("prompt", prompt="请输入预设\n输入取消 或 算了可以终止创建")
async def __spark_addprompt____(
    event: Event, state: T_State, infos: str = ArgStr("prompt")
):
    global spark_persistor
    if infos in ["取消", "算了"]:
        await spark_addprompt.finish("终止添加")
    infos = infos.split(" ")
    name = state["key"]
    prompt = infos[0]
    # # 将更新后的字典写回到JSON文件中
    spark_persistor.prompts_dict[name] = prompt
    spark_persistor.save()
    await spark_addprompt.finish("成功添加prompt")


######################################################
spark_removeprompt = on_command("删除预设", aliases={"rp"}, priority=4, block=False)


@spark_removeprompt.handle()
async def __spark_removeprompt__(event: Event):
    global spark_persistor
    user_id = str(event.user_id)
    if user_id not in spark_persistor.superusers:
        await spark_removeprompt.finish("你不是管理员哦")
    else:
        str_prompts = str()
        i = 1
        for key, value in spark_persistor.prompts_dict.items():
            str_prompts += f"************************\n{i}:预设名称：{key}\n预设内容：{value}\n"
            i += 1
        await spark_removeprompt.send(f"当前预设有：\n{str_prompts}")


@spark_removeprompt.got("name", prompt="请输入要删除的预设名称\n输入取消 或 算了可以终止创建")
async def __spark_removeprompt____(event: Event, infos: str = ArgStr("name")):
    if infos in ["取消", "算了"]:
        await spark_removeprompt.finish("终止删除")
    infos = infos.split(" ")
    if len(infos) != 1 or infos[0] not in spark_persistor.prompts_dict:
        await spark_removeprompt.reject("你输入的信息有误，请检查后重新输入")
    if is_auto_prompt(infos[0]):
        prompt_no = is_auto_prompt(infos[0])
        await spark_removeprompt.finish(f"不能删除{prompt_no}自动创建机器人时指定的预设")

    del spark_persistor.prompts_dict[infos[0]]
    spark_persistor.save()
    await spark_removeprompt.finish(f"成功删除预设{infos[0]}")


spark_removeprompt = on_command(
    "bot列表", aliases={"botlist", "bl"}, priority=4, block=False
)


@spark_removeprompt.handle()
async def __spark_removeprompt__(event: Event, matcher: Matcher):
    current_userinfo, current_userdata = get_user_info_and_data(
        event,
    )
    try:
        gw_bots = list(gptweb_persistor.user_dict[current_userinfo]["all"].keys())
        gw_bot_str = "gpt_web机器人有:\n"
        for i in range(len(gw_bots)):
            gw_bot_str += f"    {i+1}:{gw_bots[i]}\n"
    except:
        gw_bot_str = "没有可用的gpt_web机器人\n"
    poe_bots = list(poe_persistor.user_dict[current_userinfo]["all"].keys())
    try:
        poe_bot_str = "poe机器人有:\n"
        for i in range(len(poe_bots)):
            poe_bot_str += f"    {i+1}:{poe_bots[i]}\n"
    except:
        poe_bot_str = "没有可用的poe机器人\n"
    msg = "所有机器人信息如下:\n\n" + gw_bot_str + poe_bot_str
    await matcher.finish(reply_out(event, msg))


test = on_command("md2pic", priority=4, block=False)


@test.handle()
async def test_(
    matcher: Matcher,
    event: Event,
    state: T_State,
    args: Message = CommandArg(),
):
    text = str(args[0])
    pic = await md_to_pic(text)
    await matcher.finish(reply_out(event, pic))


from typing import Union
import uuid, asyncio
from nonebot import logger, require

require("nonebot_plugin_guild_patch")
from nonebot_plugin_guild_patch import GuildMessageEvent
from nonebot.adapters.onebot.v11 import Message, MessageEvent, MessageSegment
from .config import gptweb_persistor
from ..common.render.render import md_to_pic
from ..common.common_func import get_url


def reply_out(
    event: MessageEvent, content: Union[MessageSegment, Message, str, bytes]
) -> Message:
    """返回一个回复消息"""
    if isinstance(event, GuildMessageEvent):
        return Message(content)
    if type(content) == bytes:
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    if content[0:9] == "base64://":
        return MessageSegment.reply(event.message_id) + MessageSegment.image(content)
    return MessageSegment.reply(event.message_id) + content


def random_uuid4():
    return str(uuid.uuid4())


def is_useable(event, mode=gptweb_persistor.mode):
    if gptweb_persistor.session_token:
        if mode == "black":
            try:
                if str(event.user_id) in gptweb_persistor.blacklist:
                    logger.warning(f"黑名单用户:{str(event.user_id)},跳过")
                    return False
            except:
                pass
            try:
                if str(event.group_id) in gptweb_persistor.blacklist:
                    logger.warning(f"黑名单群聊:{str(event.group_id)},跳过")
                    return False
            except:
                pass
            return True
        elif mode == "white":
            try:
                if str(event.user_id) in gptweb_persistor.whitelist:
                    return True
            except:
                pass
            try:
                if str(event.group_id) in gptweb_persistor.whitelist:
                    return True
            except:
                pass
            logger.warning("用户或群聊不在白名单内，跳过")
            return False
    else:
        logger.warning("没有配置gpt_web的cookie，无法使用，跳过")
        return False


async def delete_messages(bot, user_id: str, dict_list: dict):
    await asyncio.sleep(1)
    if user_id in dict_list:
        if isinstance(dict_list[user_id], list):
            for eachmsg in dict_list[user_id]:
                await bot.delete_msg(message_id=eachmsg["message_id"])
            del dict_list[user_id]
        else:
            await bot.delete_msg(message_id=dict_list[user_id]["message_id"])
    else:
        for eachmsg in dict_list:
            await bot.delete_msg(message_id=eachmsg["message_id"])
        del dict_list


async def sendmsg(msg, matcher, event):
    if gptweb_persistor.pic_able is not None:
        if gptweb_persistor.pic_able == "True":
            pic = await md_to_pic(msg)
            if gptweb_persistor.url_able == "True":
                url = await get_url(msg)
                msgid_container = await matcher.send(
                    reply_out(event, pic) + MessageSegment.text(url)
                )
            else:
                msgid_container = await matcher.send(reply_out(event, pic))
        else:
            msgid_container = await matcher.send(reply_out(event, pic))
    else:
        if len(msg) > gptweb_persistor.num_limit:
            pic = await md_to_pic(msg)
            if gptweb_persistor.url_able == "True":
                url = await get_url(msg)
                msgid_container = await matcher.send(
                    reply_out(event, pic) + MessageSegment.text(url)
                )
            else:
                msgid_container = await matcher.send(reply_out(event, pic))
        else:
            msgid_container = await matcher.send(reply_out(event, msg))
    return msgid_container



from typing import Dict, List, Optional
import nonebot
from nonebot.adapters.onebot.v11 import MessageEvent
from pathlib import Path
from pydantic import BaseModel,Extra
from EdgeGPT import Chatbot
from ..common.config import spark_persistor

class UserInfo(BaseModel):
    platform: str
    user_id: int

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.platform, self.user_id))

    @classmethod
    def from_dict(cls, data: dict) -> "UserInfo":
        return cls(**data)

    def to_string(self) -> str:
        return f"{self.platform}-{self.user_id}"

    @classmethod
    def from_string(cls, s: str) -> "UserInfo":
        platform, user_id = s.split("-")
        return cls(platform=platform, user_id=int(user_id))


class Sender(BaseModel, extra=Extra.ignore):
    user_id: int
    user_name: str

    def __hash__(self):
        return hash((self.user_id, self.user_name))

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "Sender":
        return cls(**data)


class UserData(BaseModel, arbitrary_types_allowed=True):
    sender: Sender
    chatbot: Optional[Chatbot] = None
    chatmode: str = "1"
    last_reply_message_id: str = 0
    last_suggests: list = []
    is_waiting: bool = False

    def __hash__(self):
        return hash((self.sender))

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "UserData":
        return cls(**data)


# 储存newbing的临时数据
class NewBingTemper(BaseModel):
    # 储存以用户为主体的信息，用户的info和data，只会用到单项查找
    user_data_dict: dict[UserInfo, UserData] = {}


def set_userdata(
    event: MessageEvent, user_data_dict: dict[UserInfo, UserData]
) -> UserData:
    """
    接受一个event和一个user_data_dict(全局的，所有用户的),在user_data_dict匹配(无则初次设置)用户的过往使用信息
    返回UserInfo和匹配后的(无则新建的)对应用户的current_user_dict结构为Dict[UserInfo, UserData]
    """
    user_info = UserInfo(platform="qq", user_id=event.user_id)
    user_data = UserData(
        sender=Sender(
            user_id=event.user_id,
            user_name=event.sender.nickname or "unknown",
        )
    )
    return user_info, user_data_dict.setdefault(user_info, user_data)


# 储存newbing的数据,不需要本地保存,每次启动从env读取覆盖即可
class NerBingPersistor(BaseModel):
    cookie_path_: str = str(Path()) + "/data/spark_gpt/newbing_cookie.json"
    pic_able: str = None
    url_able: str = "True"
    suggest_able: str = "True"
    num_limit: int = 350
    blacklist: List[str] = []
    whitelist: List[str] = []
    proxy: str = ""
    mode: str = "black"

    def __init__(self, **data):
        super().__init__(**data)

        get_config = nonebot.get_driver().config

        self.blacklist = (
            get_config.newbing_blacklist
            if hasattr(get_config, "newbing_blacklist")
            else spark_persistor.blacklist
        )
        self.whitelist = (
            get_config.newbing_whitelist
            if hasattr(get_config, "newbing_whitelist")
            else spark_persistor.whitelist
        )
        self.mode = (
            get_config.newbing_mode
            if hasattr(get_config, "newbing_mode")
            and get_config.newbing_mode in ["white", "black"]
            else spark_persistor.mode
        )
        self.proxy = getattr(get_config, "newbing_proxy", self.proxy)
        self.pic_able = getattr(get_config, "newbing_picable", spark_persistor.pic_able)
        self.url_able = getattr(get_config, "newbing_urlable", spark_persistor.url_able)
        self.suggest_able = getattr(
            get_config, "newbing_suggestable", spark_persistor.suggest_able
        )
        self.num_limit = int(getattr(get_config, "newbing_limit", spark_persistor.num_limit))

# 这里的配置即使在多平台，也应只初始化一次
newbing_persistor = NerBingPersistor()



import json
from typing import Dict, Optional, Tuple
from bidict import bidict
import nonebot
from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageEvent
from pathlib import Path
from pydantic import BaseModel, Extra
from ..common.config import spark_persistor
import json


class BotInfo(BaseModel):
    # BotInfo本身储存就决定了它是单平台单来源的
    # 这里预留一些东西只是留作可能的修改使用
    nickname: Optional[str]
    # 此处的truename用uuid4来随机生成
    truename: Optional[str]
    parentname: Optional[str]
    source: Optional[str]
    model: Optional[str]
    num_users: Optional[int]
    prompt_nickname: Optional[str]
    prompt: Optional[str]
    # 这里做owner主要是用来hash，以区分不同用户的可能重名的bot
    owner: Optional[str]

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.nickname, self.owner))

    @classmethod
    def from_dict(cls, data: dict) -> "BotInfo":
        return cls(**data)


class UserInfo(BaseModel):
    platform: str
    user_id: int

    def to_dict(self) -> dict:
        return self.dict()

    def __hash__(self):
        return hash((self.platform, self.user_id))

    @classmethod
    def from_dict(cls, data: dict) -> "UserInfo":
        return cls(**data)

    def to_string(self) -> str:
        return f"{self.platform}-{self.user_id}"

    @classmethod
    def from_string(cls, s: str) -> "UserInfo":
        platform, user_id = s.split("-")
        return cls(platform=platform, user_id=int(user_id))


class Sender(BaseModel, extra=Extra.ignore):
    user_id: int
    user_name: str

    def __hash__(self):
        return hash((self.user_id, self.user_name))

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "Sender":
        return cls(**data)


class UserData(BaseModel):
    sender: Sender
    last_reply_message_id: Dict[str, int] = {}
    is_waiting: bool = False

    def __hash__(self):
        return hash((self.sender, self.last_reply_message_id))

    def to_dict(self) -> Dict:
        return self.dict()

    @classmethod
    def from_dict(cls, data: Dict) -> "UserData":
        return cls(**data)


# 储存GPT_web的临时数据
class GPT_webTemper(BaseModel):
    # 储存以用户为主体的信息，用户的info和data，只会用到单项查找
    user_data_dict: dict[UserInfo, UserData] = {}
    # 储存以bot为主体的信息，所有bot的msgid，和对应的bot信息,用bidict来方便操作唯一的键值对
    msg_bot_bidict: bidict[int, BotInfo] = bidict()


def set_userdata(
    event: MessageEvent, user_data_dict: dict[UserInfo, UserData]
) -> Tuple[UserInfo, UserData]:
    """
    接受一个event和一个user_data_dict(全局的，所有用户的),在user_data_dict匹配(无则初次设置)用户的过往使用信息
    返回UserInfo和匹配后的(无则新建的)对应用户的current_user_dict结构为[UserInfo, UserData]
    """
    user_info = UserInfo(platform="qq", user_id=event.user_id)
    user_data = UserData(
        sender=Sender(
            user_id=event.user_id,
            user_name=event.sender.nickname or "<未知的的用户名>",
        )
    )
    return user_info, user_data_dict.setdefault(user_info, user_data)


def get_user_info_and_data(event: MessageEvent) -> Tuple[UserInfo, UserData]:
    """
    接受一个 event ，返回一个包含 UserInfo 和 UserData 的元组。
    仅仅将构造好的 UserInfo 和 UserData 构建成一个一个元组并返回。
    用来初次生成UserInfo和UserData
    """
    return (
        UserInfo(platform="qq", user_id=event.user_id),
        UserData(
            sender=Sender(
                user_id=event.user_id,
                user_name=event.sender.nickname or "<未知的的用户名>",
            )
        ),
    )


# 储存GPT_Web的持久数据,提供一键保存和加载功能
class GPT_WebPersistor(BaseModel):
    path_: str = str(Path()) + "/data/spark_gpt/gpt_web_data.json"
    session_token_path: str = str(Path()) + "/data/spark_gpt/gpt_web_session_token.txt"
    session_token: str = ""
    pic_able: str = None
    url_able: str = "True"
    num_limit: int = 350
    model: str = "text-davinci-002-render-sha"
    mode: str = "black"
    api_url: str = "https://chat.loli.vet/"
    proxy:str = None
    cookie_dict: dict = {}
    superusers = []
    blacklist = []
    whitelist = []
    # user_dict: Dict[UserInfo, Dict["all" or "now", Dict[nickname, BotInfo]]] = {}
    user_dict: Dict[UserInfo, Dict[str, Dict[str, BotInfo]]] = {}
    auto_prompt: str = "默认"

    def __init__(self, **data):
        super().__init__(**data)
        try:
            saved_data = self.load()
            self.__dict__.update(saved_data)
        except:
            pass
        get_config = nonebot.get_driver().config
        self.superusers = (
            get_config.gpt_web_superusers
            if hasattr(get_config, "gpt_web_superusers")
            else spark_persistor.superusers
        )
        self.blacklist = (
            get_config.gpt_web_blacklist
            if hasattr(get_config, "gpt_web_blacklist")
            else spark_persistor.blacklist
        )
        self.whitelist = (
            get_config.gpt_web_whitelist
            if hasattr(get_config, "gpt_web_whitelist")
            else spark_persistor.whitelist
        )
        self.mode = (
            get_config.gpt_web_mode
            if hasattr(get_config, "gpt_web_mode")
            and get_config.gpt_web_mode in ["white", "black"]
            else spark_persistor.mode
        )
        self.pic_able = getattr(get_config, "gpt_web_picable", spark_persistor.pic_able)
        self.url_able = getattr(get_config, "gpt_web_urlable", spark_persistor.url_able)
        self.num_limit = int(
            getattr(get_config, "gpt_web_limit", spark_persistor.num_limit)
        )
        self.model = getattr(get_config, "gpt_web_model", self.model)
        self.api_url = getattr(get_config, "gpt_web_api_url", self.api_url)
        self.proxy = getattr(get_config, "gpt_web_proxy", None)
        try:
            with open(self.session_token_path, "r") as f:
                self.session_token = f.read().strip()
        except:
            logger.error("gpt_web 的 session_token.txt文件有误，请检查")
        self.save()

    def save(self):
        Path(self.path_).parent.mkdir(parents=True, exist_ok=True)
        new_user_dict = {}
        user_dict = self.user_dict
        for user, bot_dict in user_dict.items():
            new_user_dict[user.to_string()] = {}
            for class_name, bot_info_dict in bot_dict.items():
                new_user_dict[user.to_string()][class_name] = {}
                for nickname, bot_info in bot_info_dict.items():
                    new_user_dict[user.to_string()][class_name][
                        nickname
                    ] = bot_info.to_dict()
        data = self.__dict__.copy()
        data["user_dict"] = new_user_dict
        with open(self.path_, "w", encoding="utf-8") as f:
            json.dump(data, f, ensure_ascii=False)

    def load(self):
        with open(self.path_, "r", encoding="utf-8") as f:
            data = json.load(f)
        user_dict = data.get("user_dict", {})
        new_user_dict = {}
        for user_info_s, bot_infos in user_dict.items():
            userinfo = UserInfo.from_string(user_info_s)
            new_bots_infos = {}
            for class_name, bots in bot_infos.items():
                new_bots_infos[class_name] = {}
                for nickname, bot in bots.items():
                    new_bots_infos[class_name][nickname] = BotInfo.from_dict(bot)
            new_user_dict[userinfo] = new_bots_infos
        data["user_dict"] = new_user_dict
        return data


# 这里的配置即使在多平台，也应只初始化一次
gptweb_persistor = GPT_WebPersistor()



from os import getcwd
import re
from typing import Union, Literal
from pathlib import Path
import jinja2
import aiofiles
import markdown
import pymdownx
from nonebot.log import logger
from ...poe.pwframework import pwfw

TEMPLATES_PATH = str(Path(__file__).parent / "templates")

env = jinja2.Environment(
    extensions=["jinja2.ext.loopcontrols"],
    loader=jinja2.FileSystemLoader(TEMPLATES_PATH),
    enable_async=True,
)


async def md_to_pic(
    md: str = "",
    md_path: str = "",
    css_path: str = "",
    type: Literal["jpeg", "png"] = "png",
    quality: Union[int, None] = None,
) -> bytes:
    """markdown 转 图片
    Args:
        md (str, optional): markdown 格式文本
        md_path (str, optional): markdown 文件路径
        css_path (str,  optional): css文件路径. Defaults to None.
        type (Literal["jpeg", "png"]): 图片类型, 默认 png
        quality (int, optional): 图片质量 0-100 当为`png`时无效
    Returns:
        bytes: 图片, 可直接发送
    """
    template = env.get_template("markdown.html")
    if not md:
        if md_path:
            md = await read_file(md_path)
        else:
            raise Exception("必须输入 md 或 md_path")
    # logger.debug(md)
    md = markdown.markdown(
        md,
        extensions=[
            "pymdownx.tasklist",
            "tables",
            "fenced_code",
            "codehilite",
            "mdx_math",
            "pymdownx.tilde",
        ],
        extension_configs={"mdx_math": {"enable_dollar_delimiter": True}},
    )

    # logger.debug(md)
    extra = ""
    if "math/tex" in md:
        katex_css = await read_tpl("katex/katex.min.b64_fonts.css")
        katex_js = await read_tpl("katex/katex.min.js")
        mathtex_js = await read_tpl("katex/mathtex-script-type.min.js")
        extra = (
            f'<style type="text/css">{katex_css}</style>'
            f"<script defer>{katex_js}</script>"
            f"<script defer>{mathtex_js}</script>"
        )

    if css_path:
        css = await read_file(css_path)
    else:
        css = await read_tpl("github-markdown-light.css") + await read_tpl(
            "pygments-default.css"
        )

    return await html_to_pic(
        template_path=f"file://{css_path if css_path else TEMPLATES_PATH}",
        html=await template.render_async(md=md, css=css, extra=extra),
        type=type,
        quality=quality,
    )


async def html_to_pic(
    html: str,
    wait: int = 0,
    template_path: str = f"file://{getcwd()}",
    type: Literal["jpeg", "png"] = "png",
    quality: Union[int, None] = None,
) -> bytes:
    """html转图片

    Args:
        html (str): html文本
        wait (int, optional): 等待时间. Defaults to 0.
        template_path (str, optional): 模板路径 如 "file:///path/to/template/"
        type (Literal["jpeg", "png"]): 图片类型, 默认 png
        quality (int, optional): 图片质量 0-100 当为`png`时无效
        device_scale_factor: 缩放比例,类型为float,值越大越清晰(真正想让图片清晰更优先请调整此选项)
        **kwargs: 传入 page 的参数

    Returns:
        bytes: 图片, 可直接发送
    """

    # logger.debug(f"html:\n{html}")
    if "file:" not in template_path:
        raise Exception("template_path 应该为 file:///path/to/template")
    page = await pwfw.new_page()
    await page.set_viewport_size({"width": 700, "height": 10})
    await page.goto(template_path)
    await page.set_content(html, wait_until="networkidle")
    await page.wait_for_timeout(wait)
    img_raw = await page.screenshot(
        full_page=True,
        type=type,
        quality=quality,
    )
    await page.close()
    return img_raw


async def read_file(path: str) -> str:
    async with aiofiles.open(path, mode="r") as f:
        return await f.read()


async def read_tpl(path: str) -> str:
    return await read_file(f"{TEMPLATES_PATH}/{path}")
